@@ -164,6 +164,7 @@ sway_cmd cmd_resize;
 sway_cmd cmd_scratchpad;
 sway_cmd cmd_seamless_mouse;
 sway_cmd cmd_set;
+sway_cmd cmd_shortcuts_inhibitor;
 sway_cmd cmd_show_marks;
 sway_cmd cmd_smart_borders;
 sway_cmd cmd_smart_gaps;
@@ -266,6 +267,7 @@ sway_cmd input_cmd_xkb_rules;
 sway_cmd input_cmd_xkb_switch_layout;
 sway_cmd input_cmd_xkb_variant;
 
+sway_cmd output_cmd_adaptive_sync;
 sway_cmd output_cmd_background;
 sway_cmd output_cmd_disable;
 sway_cmd output_cmd_dpms;
@@ -287,6 +289,7 @@ sway_cmd seat_cmd_idle_inhibit;
 sway_cmd seat_cmd_idle_wake;
 sway_cmd seat_cmd_keyboard_grouping;
 sway_cmd seat_cmd_pointer_constraint;
+sway_cmd seat_cmd_shortcuts_inhibitor;
 sway_cmd seat_cmd_xcursor_theme;
 
 sway_cmd cmd_ipc_cmd;
