@@ -32,6 +32,7 @@ struct sway_cursor {
 
 	struct wlr_pointer_constraint_v1 *active_constraint;
 	pixman_region32_t confine; // invalid if active_constraint == NULL
+	bool active_confine_requires_warp;
 
 	struct wlr_pointer_gestures_v1 *pointer_gestures;
 	struct wl_listener pinch_begin;
@@ -50,11 +51,14 @@ struct sway_cursor {
 	struct wl_listener touch_down;
 	struct wl_listener touch_up;
 	struct wl_listener touch_motion;
+	bool simulating_pointer_from_touch;
+	int32_t pointer_touch_id;
 
 	struct wl_listener tool_axis;
 	struct wl_listener tool_tip;
 	struct wl_listener tool_proximity;
 	struct wl_listener tool_button;
+	bool simulating_pointer_from_tool_tip;
 	uint32_t tool_buttons;
 
 	struct wl_listener request_set_cursor;
@@ -84,8 +88,10 @@ struct sway_cursor *sway_cursor_create(struct sway_seat *seat);
  */
 void cursor_rebase(struct sway_cursor *cursor);
 void cursor_rebase_all(void);
+void cursor_update_image(struct sway_cursor *cursor, struct sway_node *node);
 
-void cursor_handle_activity(struct sway_cursor *cursor);
+void cursor_handle_activity(struct sway_cursor *cursor,
+		struct wlr_input_device *device);
 void cursor_unhide(struct sway_cursor *cursor);
 int cursor_get_timeout(struct sway_cursor *cursor);
 
