@@ -2,6 +2,7 @@
 #define _SWAY_INPUT_INPUT_MANAGER_H
 #include <libinput.h>
 #include <wlr/types/wlr_input_inhibitor.h>
+#include <wlr/types/wlr_keyboard_shortcuts_inhibit_v1.h>
 #include <wlr/types/wlr_virtual_keyboard_v1.h>
 #include <wlr/types/wlr_virtual_pointer_v1.h>
 #include "sway/server.h"
@@ -13,6 +14,7 @@ struct sway_input_device {
 	struct wlr_input_device *wlr_device;
 	struct wl_list link;
 	struct wl_listener device_destroy;
+	bool is_virtual;
 };
 
 struct sway_input_manager {
@@ -20,12 +22,14 @@ struct sway_input_manager {
 	struct wl_list seats;
 
 	struct wlr_input_inhibit_manager *inhibit;
+	struct wlr_keyboard_shortcuts_inhibit_manager_v1 *keyboard_shortcuts_inhibit;
 	struct wlr_virtual_keyboard_manager_v1 *virtual_keyboard;
 	struct wlr_virtual_pointer_manager_v1 *virtual_pointer;
 
 	struct wl_listener new_input;
 	struct wl_listener inhibit_activate;
 	struct wl_listener inhibit_deactivate;
+	struct wl_listener keyboard_shortcuts_inhibit_new_inhibitor;
 	struct wl_listener virtual_keyboard_new;
 	struct wl_listener virtual_pointer_new;
 };
@@ -40,9 +44,11 @@ void input_manager_configure_xcursor(void);
 
 void input_manager_apply_input_config(struct input_config *input_config);
 
+void input_manager_configure_all_inputs(void);
+
 void input_manager_reset_input(struct sway_input_device *input_device);
 
-void input_manager_reset_all_inputs();
+void input_manager_reset_all_inputs(void);
 
 void input_manager_apply_seat_config(struct seat_config *seat_config);
 
