@@ -1,11 +1,14 @@
 #ifndef _SWAY_INPUT_SEAT_H
 #define _SWAY_INPUT_SEAT_H
 
+#include <wlr/types/wlr_keyboard_shortcuts_inhibit_v1.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include <wlr/types/wlr_seat.h>
 #include <wlr/util/edges.h>
 #include "sway/config.h"
 #include "sway/input/input-manager.h"
+#include "sway/input/tablet.h"
+#include "sway/input/text_input.h"
 
 struct sway_seat;
 
@@ -13,10 +16,15 @@ struct sway_seatop_impl {
 	void (*button)(struct sway_seat *seat, uint32_t time_msec,
 			struct wlr_input_device *device, uint32_t button,
 			enum wlr_button_state state);
-	void (*motion)(struct sway_seat *seat, uint32_t time_msec,
+	void (*pointer_motion)(struct sway_seat *seat, uint32_t time_msec,
 			double dx, double dy);
-	void (*axis)(struct sway_seat *seat, struct wlr_event_pointer_axis *event);
+	void (*pointer_axis)(struct sway_seat *seat,
+			struct wlr_event_pointer_axis *event);
 	void (*rebase)(struct sway_seat *seat, uint32_t time_msec);
+	void (*tablet_tool_motion)(struct sway_seat *seat,
+			struct sway_tablet_tool *tool, uint32_t time_msec, double dx, double dy);
+	void (*tablet_tool_tip)(struct sway_seat *seat, struct sway_tablet_tool *tool,
+			uint32_t time_msec, enum wlr_tablet_tool_tip_state state);
 	void (*end)(struct sway_seat *seat);
 	void (*unref)(struct sway_seat *seat, struct sway_container *con);
 	void (*render)(struct sway_seat *seat, struct sway_output *output,
@@ -56,6 +64,12 @@ struct sway_drag_icon {
 	struct wl_listener destroy;
 };
 
+struct sway_drag {
+	struct sway_seat *seat;
+	struct wlr_drag *wlr_drag;
+	struct wl_listener destroy;
+};
+
 struct sway_seat {
 	struct wlr_seat *wlr_seat;
 	struct sway_cursor *cursor;
@@ -85,6 +99,8 @@ struct sway_seat {
 
 	list_t *deferred_bindings; // struct sway_binding
 
+	struct sway_input_method_relay im_relay;
+
 	struct wl_listener focus_destroy;
 	struct wl_listener new_node;
 	struct wl_listener request_start_drag;
@@ -94,16 +110,28 @@ struct sway_seat {
 
 	struct wl_list devices; // sway_seat_device::link
 	struct wl_list keyboard_groups; // sway_keyboard_group::link
+	struct wl_list keyboard_shortcuts_inhibitors;
+				// sway_keyboard_shortcuts_inhibitor::link
 
 	struct wl_list link; // input_manager::seats
 };
 
 struct sway_pointer_constraint {
+	struct sway_cursor *cursor;
 	struct wlr_pointer_constraint_v1 *constraint;
 
+	struct wl_listener set_region;
 	struct wl_listener destroy;
 };
 
+struct sway_keyboard_shortcuts_inhibitor {
+	struct wlr_keyboard_shortcuts_inhibitor_v1 *inhibitor;
+
+	struct wl_listener destroy;
+
+	struct wl_list link; // sway_seat::keyboard_shortcuts_inhibitors
+};
+
 struct sway_seat *seat_create(const char *seat_name);
 
 void seat_destroy(struct sway_seat *seat);
@@ -206,6 +234,9 @@ bool seat_is_input_allowed(struct sway_seat *seat, struct wlr_surface *surface);
 
 void drag_icon_update_position(struct sway_drag_icon *icon);
 
+enum wlr_edges find_resize_edge(struct sway_container *cont,
+		struct wlr_surface *surface, struct sway_cursor *cursor);
+
 void seatop_begin_default(struct sway_seat *seat);
 
 void seatop_begin_down(struct sway_seat *seat, struct sway_container *con,
@@ -241,10 +272,18 @@ void seatop_button(struct sway_seat *seat, uint32_t time_msec,
 /**
  * dx and dy are distances relative to previous position.
  */
-void seatop_motion(struct sway_seat *seat, uint32_t time_msec,
+void seatop_pointer_motion(struct sway_seat *seat, uint32_t time_msec,
 		double dx, double dy);
 
-void seatop_axis(struct sway_seat *seat, struct wlr_event_pointer_axis *event);
+void seatop_pointer_axis(struct sway_seat *seat,
+		struct wlr_event_pointer_axis *event);
+
+void seatop_tablet_tool_tip(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec,
+		enum wlr_tablet_tool_tip_state state);
+
+void seatop_tablet_tool_motion(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec, double dx, double dy);
 
 void seatop_rebase(struct sway_seat *seat, uint32_t time_msec);
 
@@ -269,4 +308,19 @@ void seatop_render(struct sway_seat *seat, struct sway_output *output,
 
 bool seatop_allows_set_cursor(struct sway_seat *seat);
 
+/**
+ * Returns the keyboard shortcuts inhibitor that applies to the given surface
+ * or NULL if none exists.
+ */
+struct sway_keyboard_shortcuts_inhibitor *
+keyboard_shortcuts_inhibitor_get_for_surface(const struct sway_seat *seat,
+		const struct wlr_surface *surface);
+
+/**
+ * Returns the keyboard shortcuts inhibitor that applies to the currently
+ * focused surface of a seat or NULL if none exists.
+ */
+struct sway_keyboard_shortcuts_inhibitor *
+keyboard_shortcuts_inhibitor_get_for_focused_surface(const struct sway_seat *seat);
+
 #endif
