@@ -2,6 +2,7 @@
 #define _SWAY_IPC_SERVER_H
 #include <sys/socket.h>
 #include "sway/config.h"
+#include "sway/input/input-manager.h"
 #include "sway/tree/container.h"
 #include "ipc.h"
 
@@ -19,5 +20,6 @@ void ipc_event_bar_state_update(struct bar_config *bar);
 void ipc_event_mode(const char *mode, bool pango);
 void ipc_event_shutdown(const char *reason);
 void ipc_event_binding(struct sway_binding *binding);
+void ipc_event_input(const char *change, struct sway_input_device *device);
 
 #endif
