@@ -7,11 +7,15 @@
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_data_device.h>
+#include <wlr/types/wlr_input_method_v2.h>
+#include <wlr/types/wlr_foreign_toplevel_management_v1.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include <wlr/types/wlr_output_management_v1.h>
+#include <wlr/types/wlr_output_power_management_v1.h>
 #include <wlr/types/wlr_presentation_time.h>
 #include <wlr/types/wlr_relative_pointer_v1.h>
 #include <wlr/types/wlr_server_decoration.h>
+#include <wlr/types/wlr_text_input_v3.h>
 #include <wlr/types/wlr_xdg_shell.h>
 #include "config.h"
 #include "list.h"
@@ -26,6 +30,8 @@ struct sway_server {
 
 	struct wlr_backend *backend;
 	struct wlr_backend *noop_backend;
+	// secondary headless backend used for creating virtual outputs on-the-fly
+	struct wlr_backend *headless_backend;
 
 	struct wlr_compositor *compositor;
 	struct wl_listener compositor_new_surface;
@@ -73,12 +79,18 @@ struct sway_server {
 	struct wl_listener output_manager_apply;
 	struct wl_listener output_manager_test;
 
+	struct wlr_output_power_manager_v1 *output_power_manager_v1;
+	struct wl_listener output_power_manager_set_mode;
+	struct wlr_input_method_manager_v2 *input_method;
+	struct wlr_text_input_manager_v3 *text_input;
+	struct wlr_foreign_toplevel_manager_v1 *foreign_toplevel_manager;
+
 	size_t txn_timeout_ms;
 	list_t *transactions;
 	list_t *dirty_nodes;
 };
 
-struct sway_server server;
+extern struct sway_server server;
 
 struct sway_debug {
 	bool noatomic;         // Ignore atomic layout updates
@@ -92,7 +104,7 @@ struct sway_debug {
 	} damage;
 };
 
-struct sway_debug debug;
+extern struct sway_debug debug;
 
 /* Prepares an unprivileged server_init by performing all privileged operations in advance */
 bool server_privileged_prepare(struct sway_server *server);
