@@ -105,7 +105,6 @@ struct sway_container {
 	// refuses to resize to the content dimensions then it can be smaller.
 	// These are in layout coordinates.
 	double surface_x, surface_y;
-	double surface_width, surface_height;
 
 	enum sway_fullscreen_mode fullscreen_mode;
 
@@ -167,7 +166,7 @@ struct sway_container *container_find_child(struct sway_container *container,
 		bool (*test)(struct sway_container *view, void *data), void *data);
 
 /**
- * Find a container at the given coordinates. Returns the the surface and
+ * Find a container at the given coordinates. Returns the surface and
  * surface-local coordinates of the given layout coordinates if the container
  * is a view and the view contains a surface at those coordinates.
  */
@@ -273,6 +272,13 @@ void container_set_fullscreen(struct sway_container *con,
  */
 void container_fullscreen_disable(struct sway_container *con);
 
+/**
+ * Walk up the container tree branch starting at the given container, and return
+ * its earliest ancestor.
+ */
+struct sway_container *container_toplevel_ancestor(
+		struct sway_container *container);
+
 /**
  * Return true if the container is floating, or a child of a floating split
  * container.
