@@ -55,6 +55,15 @@ struct sway_view_impl {
 	void (*destroy)(struct sway_view *view);
 };
 
+struct sway_saved_buffer {
+	struct wlr_client_buffer *buffer;
+	int x, y;
+	int width, height;
+	enum wl_output_transform transform;
+	struct wlr_fbox source_box;
+	struct wl_list link; // sway_view::saved_buffers
+};
+
 struct sway_view {
 	enum sway_view_type type;
 	const struct sway_view_impl *impl;
@@ -65,9 +74,6 @@ struct sway_view {
 
 	pid_t pid;
 
-	double saved_x, saved_y;
-	int saved_width, saved_height;
-
 	// The size the view would want to be if it weren't tiled.
 	// Used when changing a view from tiled to floating.
 	int natural_width, natural_height;
@@ -80,8 +86,7 @@ struct sway_view {
 	bool allow_request_urgent;
 	struct wl_event_source *urgent_timer;
 
-	struct wlr_buffer *saved_buffer;
-	int saved_buffer_width, saved_buffer_height;
+	struct wl_list saved_buffers; // sway_saved_buffer::link
 
 	// The geometry for whatever the client is committing, regardless of
 	// transaction state. Updated on every commit.
@@ -91,6 +96,10 @@ struct sway_view {
 	// when a transaction is applied.
 	struct wlr_box saved_geometry;
 
+	struct wlr_foreign_toplevel_handle_v1 *foreign_toplevel;
+	struct wl_listener foreign_activate_request;
+	struct wl_listener foreign_close_request;
+
 	bool destroying;
 
 	list_t *executed_criteria; // struct criteria *
@@ -110,6 +119,8 @@ struct sway_view {
 	struct wl_listener surface_new_subsurface;
 
 	int max_render_time; // In milliseconds
+
+	enum seat_config_shortcuts_inhibit shortcuts_inhibit;
 };
 
 struct sway_xdg_shell_view {
@@ -230,12 +241,15 @@ void view_get_constraints(struct sway_view *view, double *min_width,
 uint32_t view_configure(struct sway_view *view, double lx, double ly, int width,
 	int height);
 
+bool view_inhibit_idle(struct sway_view *view);
+
 /**
- * Whether or not the view is the only visible view in its tree. If the view
- * is tiling, there may be floating views. If the view is floating, there may
- * be tiling views or views in a different floating container.
+ * Whether or not this view's most distant ancestor (possibly itself) is the
+ * only visible node in its tree. If the view is tiling, there may be floating
+ * views. If the view is floating, there may be tiling views or views in a
+ * different floating container.
  */
-bool view_is_only_visible(struct sway_view *view);
+bool view_ancestor_is_only_visible(struct sway_view *view);
 
 /**
  * Configure the view's position and size based on the container's position and
