@@ -28,7 +28,7 @@ struct i3bar_block {
 void i3bar_block_unref(struct i3bar_block *block);
 bool i3bar_handle_readable(struct status_line *status);
 enum hotspot_event_handling i3bar_block_send_click(struct status_line *status,
-		struct i3bar_block *block, int x, int y, int rx, int ry, int w, int h,
-		uint32_t button);
+		struct i3bar_block *block, double x, double y, double rx, double ry,
+		double w, double h, int scale, uint32_t button);
 
 #endif
