@@ -72,7 +72,8 @@ static bool binding_key_compare(struct sway_binding *binding_a,
 	}
 
 	uint32_t conflict_generating_flags = BINDING_RELEASE | BINDING_BORDER
-			| BINDING_CONTENTS | BINDING_TITLEBAR | BINDING_LOCKED;
+			| BINDING_CONTENTS | BINDING_TITLEBAR | BINDING_LOCKED
+			| BINDING_INHIBITED;
 	if ((binding_a->flags & conflict_generating_flags) !=
 			(binding_b->flags & conflict_generating_flags)) {
 		return false;
@@ -144,7 +145,7 @@ static struct cmd_results *identify_key(const char* name, bool first_key,
 					cmd_results_new(CMD_INVALID, message);
 				free(message);
 				return error;
-			} else if (!button) {
+			} else {
 				return cmd_results_new(CMD_INVALID, "Unknown button %s", name);
 			}
 		}
@@ -354,6 +355,8 @@ static struct cmd_results *cmd_bindsym_or_bindcode(int argc, char **argv,
 			binding->flags |= BINDING_RELEASE;
 		} else if (strcmp("--locked", argv[0]) == 0) {
 			binding->flags |= BINDING_LOCKED;
+		} else if (strcmp("--inhibited", argv[0]) == 0) {
+			binding->flags |= BINDING_INHIBITED;
 		} else if (strcmp("--whole-window", argv[0]) == 0) {
 			binding->flags |= BINDING_BORDER | BINDING_CONTENTS | BINDING_TITLEBAR;
 		} else if (strcmp("--border", argv[0]) == 0) {
@@ -370,6 +373,8 @@ static struct cmd_results *cmd_bindsym_or_bindcode(int argc, char **argv,
 			binding->input = strdup(argv[0] + strlen("--input-device="));
 		} else if (strcmp("--no-warn", argv[0]) == 0) {
 			warn = false;
+		} else if (strcmp("--no-repeat", argv[0]) == 0) {
+			binding->flags |= BINDING_NOREPEAT;
 		} else {
 			break;
 		}
@@ -603,6 +608,7 @@ void seat_execute_command(struct sway_seat *seat, struct sway_binding *binding)
 			return;
 		}
 		memcpy(deferred, binding, sizeof(struct sway_binding));
+		deferred->command = binding->command ? strdup(binding->command) : NULL;
 		list_add(seat->deferred_bindings, deferred);
 		return;
 	}
