@@ -7,6 +7,7 @@
 
 // must be in order for the bsearch
 static struct cmd_handler output_handlers[] = {
+	{ "adaptive_sync", output_cmd_adaptive_sync },
 	{ "background", output_cmd_background },
 	{ "bg", output_cmd_background },
 	{ "disable", output_cmd_disable },
@@ -103,7 +104,7 @@ struct cmd_results *cmd_output(int argc, char **argv) {
 	// If reloading, the output configs will be applied after reading the
 	// entire config and before the deferred commands so that an auto generated
 	// workspace name is not given to re-enabled outputs.
-	if (!config->reloading) {
+	if (!config->reloading && !config->validating) {
 		apply_output_config_to_outputs(output);
 		if (background) {
 			spawn_swaybg();
