@@ -154,9 +154,6 @@ void free_config(struct sway_config *config) {
 	list_free(config->no_focus);
 	list_free(config->active_bar_modifiers);
 	list_free_items_and_destroy(config->config_chain);
-	list_free(config->command_policies);
-	list_free(config->feature_policies);
-	list_free(config->ipc_policies);
 	free(config->floating_scroll_up_cmd);
 	free(config->floating_scroll_down_cmd);
 	free(config->floating_scroll_left_cmd);
@@ -241,6 +238,7 @@ static void config_defaults(struct sway_config *config) {
 	if (!(config->font = strdup("monospace 10"))) goto cleanup;
 	config->font_height = 17; // height of monospace 10
 	config->urgent_timeout = 500;
+	config->focus_on_window_activation = FOWA_URGENT;
 	config->popup_during_fullscreen = POPUP_SMART;
 	config->xwayland = XWAYLAND_MODE_LAZY;
 
@@ -308,7 +306,7 @@ static void config_defaults(struct sway_config *config) {
 
 	color_to_rgba(config->border_colors.unfocused.border, 0x333333FF);
 	color_to_rgba(config->border_colors.unfocused.background, 0x222222FF);
-	color_to_rgba(config->border_colors.unfocused.text, 0x88888888);
+	color_to_rgba(config->border_colors.unfocused.text, 0x888888FF);
 	color_to_rgba(config->border_colors.unfocused.indicator, 0x292D2EFF);
 	color_to_rgba(config->border_colors.unfocused.child_border, 0x222222FF);
 
@@ -326,11 +324,6 @@ static void config_defaults(struct sway_config *config) {
 
 	color_to_rgba(config->border_colors.background, 0xFFFFFFFF);
 
-	// Security
-	if (!(config->command_policies = create_list())) goto cleanup;
-	if (!(config->feature_policies = create_list())) goto cleanup;
-	if (!(config->ipc_policies = create_list())) goto cleanup;
-
 	// The keysym to keycode translation
 	struct xkb_rule_names rules = {0};
 	config->keysym_translation_state =
@@ -404,7 +397,7 @@ static bool load_config(const char *path, struct sway_config *config,
 		sway_log(SWAY_ERROR, "Error(s) loading config!");
 	}
 
-	return true;
+	return config->active || !config->validating || config_load_success;
 }
 
 bool load_main_config(const char *file, bool is_active, bool validating) {
@@ -454,6 +447,7 @@ bool load_main_config(const char *file, bool is_active, bool validating) {
 		}
 	}
 
+	config->user_config_path = file ? true : false;
 	config->current_config_path = path;
 	list_add(config->config_chain, real_path);
 
