@@ -39,8 +39,10 @@ void free_bar_config(struct bar_config *bar) {
 	free(bar->swaybar_command);
 	free(bar->font);
 	free(bar->separator_symbol);
-	for (int i = 0; i < bar->bindings->length; i++) {
-		free_bar_binding(bar->bindings->items[i]);
+	if (bar->bindings) {
+		for (int i = 0; i < bar->bindings->length; i++) {
+			free_bar_binding(bar->bindings->items[i]);
+		}
 	}
 	list_free(bar->bindings);
 	list_free_items_and_destroy(bar->outputs);
@@ -172,7 +174,6 @@ struct bar_config *default_bar_config(void) {
 	wl_list_init(&bar->tray_bindings);
 #endif
 
-	list_add(config->bars, bar);
 	return bar;
 cleanup:
 	free_bar_config(bar);
@@ -193,7 +194,7 @@ static void invoke_swaybar(struct bar_config *bar) {
 		sway_log_errno(SWAY_ERROR, "socketpair failed");
 		return;
 	}
-	if (!set_cloexec(sockets[0], true) || !set_cloexec(sockets[1], true)) {
+	if (!sway_set_cloexec(sockets[0], true) || !sway_set_cloexec(sockets[1], true)) {
 		return;
 	}
 
@@ -221,7 +222,7 @@ static void invoke_swaybar(struct bar_config *bar) {
 			sway_log_errno(SWAY_ERROR, "fork failed");
 			_exit(EXIT_FAILURE);
 		} else if (pid == 0) {
-			if (!set_cloexec(sockets[1], false)) {
+			if (!sway_set_cloexec(sockets[1], false)) {
 				_exit(EXIT_FAILURE);
 			}
 
