@@ -64,6 +64,7 @@ struct output_config *new_output_config(const char *name) {
 	oc->transform = -1;
 	oc->subpixel = WL_OUTPUT_SUBPIXEL_UNKNOWN;
 	oc->max_render_time = -1;
+	oc->adaptive_sync = -1;
 	return oc;
 }
 
@@ -104,6 +105,9 @@ void merge_output_config(struct output_config *dst, struct output_config *src) {
 	if (src->max_render_time != -1) {
 		dst->max_render_time = src->max_render_time;
 	}
+	if (src->adaptive_sync != -1) {
+		dst->adaptive_sync = src->adaptive_sync;
+	}
 	if (src->background) {
 		free(dst->background);
 		dst->background = strdup(src->background);
@@ -283,8 +287,28 @@ static bool phys_size_is_aspect_ratio(struct wlr_output *output) {
 #define MM_PER_INCH 25.4
 
 static int compute_default_scale(struct wlr_output *output) {
-	int width, height;
-	wlr_output_transformed_resolution(output, &width, &height);
+	struct wlr_box box = { .width = output->width, .height = output->height };
+	if (output->pending.committed & WLR_OUTPUT_STATE_MODE) {
+		switch (output->pending.mode_type) {
+		case WLR_OUTPUT_STATE_MODE_FIXED:
+			box.width = output->pending.mode->width;
+			box.height = output->pending.mode->height;
+			break;
+		case WLR_OUTPUT_STATE_MODE_CUSTOM:
+			box.width = output->pending.custom_mode.width;
+			box.height = output->pending.custom_mode.height;
+			break;
+		}
+	}
+	enum wl_output_transform transform = output->transform;
+	if (output->pending.committed & WLR_OUTPUT_STATE_TRANSFORM) {
+		transform = output->pending.transform;
+	}
+	wlr_box_transform(&box, &box, transform, box.width, box.height);
+
+	int width = box.width;
+	int height = box.height;
+
 	if (height < HIDPI_MIN_HEIGHT) {
 		return 1;
 	}
@@ -307,48 +331,46 @@ static int compute_default_scale(struct wlr_output *output) {
 	return 2;
 }
 
-bool apply_output_config(struct output_config *oc, struct sway_output *output) {
+static void queue_output_config(struct output_config *oc,
+		struct sway_output *output) {
 	if (output == root->noop_output) {
-		return false;
+		return;
 	}
 
 	struct wlr_output *wlr_output = output->wlr_output;
 
-	if (oc && !oc->enabled) {
-		// Output is configured to be disabled
-		sway_log(SWAY_DEBUG, "Disabling output %s", oc->name);
-		if (output->enabled) {
-			output_disable(output);
-			wlr_output_layout_remove(root->output_layout, wlr_output);
-		}
+	if (oc && (!oc->enabled || oc->dpms_state == DPMS_OFF)) {
+		sway_log(SWAY_DEBUG, "Turning off output %s", wlr_output->name);
 		wlr_output_enable(wlr_output, false);
-		return wlr_output_commit(wlr_output);
-	} else if (!output->enabled) {
-		// Output is not enabled. Enable it, output_enable will call us again.
-		if (!oc || oc->dpms_state != DPMS_OFF) {
-			sway_log(SWAY_DEBUG, "Enabling output %s", oc->name);
-			wlr_output_enable(wlr_output, true);
-			wlr_output_commit(wlr_output);
-		}
-		return output_enable(output, oc);
+		return;
 	}
 
-	if (!oc || oc->dpms_state != DPMS_OFF) {
-		sway_log(SWAY_DEBUG, "Turning on output %s", wlr_output->name);
-		wlr_output_enable(wlr_output, true);
+	sway_log(SWAY_DEBUG, "Turning on output %s", wlr_output->name);
+	wlr_output_enable(wlr_output, true);
 
-		if (oc && oc->width > 0 && oc->height > 0) {
-			sway_log(SWAY_DEBUG, "Set %s mode to %dx%d (%f Hz)", oc->name, oc->width,
-				oc->height, oc->refresh_rate);
-			set_mode(wlr_output, oc->width, oc->height,
-				oc->refresh_rate, oc->custom_mode == 1);
-		} else if (!wl_list_empty(&wlr_output->modes)) {
-			struct wlr_output_mode *mode = wlr_output_preferred_mode(wlr_output);
-			wlr_output_set_mode(wlr_output, mode);
-		}
-		output->current_mode = wlr_output->pending.mode;
+	if (oc && oc->width > 0 && oc->height > 0) {
+		sway_log(SWAY_DEBUG, "Set %s mode to %dx%d (%f Hz)",
+			wlr_output->name, oc->width, oc->height, oc->refresh_rate);
+		set_mode(wlr_output, oc->width, oc->height,
+			oc->refresh_rate, oc->custom_mode == 1);
+	} else if (!wl_list_empty(&wlr_output->modes)) {
+		struct wlr_output_mode *mode = wlr_output_preferred_mode(wlr_output);
+		wlr_output_set_mode(wlr_output, mode);
 	}
 
+	if (oc && (oc->subpixel != WL_OUTPUT_SUBPIXEL_UNKNOWN || config->reloading)) {
+		sway_log(SWAY_DEBUG, "Set %s subpixel to %s", oc->name,
+			sway_wl_output_subpixel_to_string(oc->subpixel));
+		wlr_output_set_subpixel(wlr_output, oc->subpixel);
+	}
+
+	if (oc && oc->transform >= 0) {
+		sway_log(SWAY_DEBUG, "Set %s transform to %d", oc->name, oc->transform);
+		wlr_output_set_transform(wlr_output, oc->transform);
+	}
+
+	// Apply the scale last before the commit, because the scale auto-detection
+	// reads the pending output size
 	float scale;
 	if (oc && oc->scale > 0) {
 		scale = oc->scale;
@@ -357,10 +379,58 @@ bool apply_output_config(struct output_config *oc, struct sway_output *output) {
 		sway_log(SWAY_DEBUG, "Auto-detected output scale: %f", scale);
 	}
 	if (scale != wlr_output->scale) {
-		sway_log(SWAY_DEBUG, "Set %s scale to %f", oc->name, scale);
+		sway_log(SWAY_DEBUG, "Set %s scale to %f", wlr_output->name, scale);
 		wlr_output_set_scale(wlr_output, scale);
 	}
 
+	if (oc && oc->adaptive_sync != -1) {
+		sway_log(SWAY_DEBUG, "Set %s adaptive sync to %d", wlr_output->name,
+			oc->adaptive_sync);
+		wlr_output_enable_adaptive_sync(wlr_output, oc->adaptive_sync == 1);
+	}
+}
+
+bool apply_output_config(struct output_config *oc, struct sway_output *output) {
+	if (output == root->noop_output) {
+		return false;
+	}
+
+	struct wlr_output *wlr_output = output->wlr_output;
+
+	// Flag to prevent the output mode event handler from calling us
+	output->enabling = (!oc || oc->enabled);
+
+	queue_output_config(oc, output);
+
+	if (!oc || oc->dpms_state != DPMS_OFF) {
+		output->current_mode = wlr_output->pending.mode;
+	}
+
+	sway_log(SWAY_DEBUG, "Committing output %s", wlr_output->name);
+	if (!wlr_output_commit(wlr_output)) {
+		// Failed to commit output changes, maybe the output is missing a CRTC.
+		// Leave the output disabled for now and try again when the output gets
+		// the mode we asked for.
+		sway_log(SWAY_ERROR, "Failed to commit output %s", wlr_output->name);
+		output->enabling = false;
+		return false;
+	}
+
+	output->enabling = false;
+
+	if (oc && !oc->enabled) {
+		sway_log(SWAY_DEBUG, "Disabling output %s", oc->name);
+		if (output->enabled) {
+			output_disable(output);
+			wlr_output_layout_remove(root->output_layout, wlr_output);
+		}
+		return true;
+	}
+
+	if (config->reloading) {
+		output_damage_whole(output);
+	}
+
 	if (oc) {
 		enum scale_filter_mode scale_filter_old = output->scale_filter;
 		switch (oc->scale_filter) {
@@ -377,31 +447,9 @@ bool apply_output_config(struct output_config *oc, struct sway_output *output) {
 		if (scale_filter_old != output->scale_filter) {
 			sway_log(SWAY_DEBUG, "Set %s scale_filter to %s", oc->name,
 				sway_output_scale_filter_to_string(output->scale_filter));
-			output_damage_whole(output);
 		}
 	}
 
-	if (oc && (oc->subpixel != WL_OUTPUT_SUBPIXEL_UNKNOWN || config->reloading)) {
-		sway_log(SWAY_DEBUG, "Set %s subpixel to %s", oc->name,
-			sway_wl_output_subpixel_to_string(oc->subpixel));
-		wlr_output_set_subpixel(wlr_output, oc->subpixel);
-		output_damage_whole(output);
-	}
-
-	if (oc && oc->transform >= 0) {
-		sway_log(SWAY_DEBUG, "Set %s transform to %d", oc->name, oc->transform);
-		wlr_output_set_transform(wlr_output, oc->transform);
-	}
-
-	sway_log(SWAY_DEBUG, "Committing output %s", wlr_output->name);
-	if (!wlr_output_commit(wlr_output)) {
-		// Failed to modeset, maybe the output is missing a CRTC. Leave the
-		// output disabled for now and try again when the output gets the mode
-		// we asked for.
-		sway_log(SWAY_ERROR, "Failed to modeset output %s", wlr_output->name);
-		return false;
-	}
-
 	// Find position for it
 	if (oc && (oc->x != -1 || oc->y != -1)) {
 		sway_log(SWAY_DEBUG, "Set %s position to %d, %d", oc->name, oc->x, oc->y);
@@ -418,10 +466,8 @@ bool apply_output_config(struct output_config *oc, struct sway_output *output) {
 	output->width = output_box->width;
 	output->height = output_box->height;
 
-	if (oc && oc->dpms_state == DPMS_OFF) {
-		sway_log(SWAY_DEBUG, "Turning off output %s", oc->name);
-		wlr_output_enable(wlr_output, false);
-		wlr_output_commit(wlr_output);
+	if (!output->enabled) {
+		output_enable(output);
 	}
 
 	if (oc && oc->max_render_time >= 0) {
@@ -430,9 +476,24 @@ bool apply_output_config(struct output_config *oc, struct sway_output *output) {
 		output->max_render_time = oc->max_render_time;
 	}
 
+	// Reconfigure all devices, since input config may have been applied before
+	// this output came online, and some config items (like map_to_output) are
+	// dependent on an output being present.
+	input_manager_configure_all_inputs();
 	return true;
 }
 
+bool test_output_config(struct output_config *oc, struct sway_output *output) {
+	if (output == root->noop_output) {
+		return false;
+	}
+
+	queue_output_config(oc, output);
+	bool ok = wlr_output_test(output->wlr_output);
+	wlr_output_rollback(output->wlr_output);
+	return ok;
+}
+
 static void default_output_config(struct output_config *oc,
 		struct wlr_output *wlr_output) {
 	oc->enabled = 1;
@@ -575,7 +636,7 @@ void apply_output_config_to_outputs(struct output_config *oc) {
 
 	struct sway_seat *seat;
 	wl_list_for_each(seat, &server.input->seats, link) {
-		wlr_seat_pointer_clear_focus(seat->wlr_seat);
+		wlr_seat_pointer_notify_clear_focus(seat->wlr_seat);
 		cursor_rebase(seat->cursor);
 	}
 }
