@@ -31,7 +31,8 @@ bool criteria_is_empty(struct criteria *criteria) {
 		&& !criteria->floating
 		&& !criteria->tiling
 		&& !criteria->urgent
-		&& !criteria->workspace;
+		&& !criteria->workspace
+		&& !criteria->pid;
 }
 
 // The error pointer is used for parsing functions, and saves having to pass it
@@ -144,6 +145,35 @@ static void find_urgent_iterator(struct sway_container *con, void *data) {
 	list_add(urgent_views, con->view);
 }
 
+static bool has_container_criteria(struct criteria *criteria) {
+	return criteria->con_mark || criteria->con_id;
+}
+
+static bool criteria_matches_container(struct criteria *criteria,
+		struct sway_container *container) {
+	if (criteria->con_mark) {
+		bool exists = false;
+		struct sway_container *con = container;
+		for (int i = 0; i < con->marks->length; ++i) {
+			if (regex_cmp(con->marks->items[i], criteria->con_mark->regex) == 0) {
+				exists = true;
+				break;
+			}
+		}
+		if (!exists) {
+			return false;
+		}
+	}
+
+	if (criteria->con_id) { // Internal ID
+		if (container->node.id != criteria->con_id) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 static bool criteria_matches_view(struct criteria *criteria,
 		struct sway_view *view) {
 	struct sway_seat *seat = input_manager_current_seat();
@@ -210,24 +240,8 @@ static bool criteria_matches_view(struct criteria *criteria,
 		}
 	}
 
-	if (criteria->con_mark) {
-		bool exists = false;
-		struct sway_container *con = view->container;
-		for (int i = 0; i < con->marks->length; ++i) {
-			if (regex_cmp(con->marks->items[i], criteria->con_mark->regex) == 0) {
-				exists = true;
-				break;
-			}
-		}
-		if (!exists) {
-			return false;
-		}
-	}
-
-	if (criteria->con_id) { // Internal ID
-		if (!view->container || view->container->node.id != criteria->con_id) {
-			return false;
-		}
+	if (!criteria_matches_container(criteria, view->container)) {
+		return false;
 	}
 
 #if HAVE_XWAYLAND
@@ -357,6 +371,12 @@ static bool criteria_matches_view(struct criteria *criteria,
 		}
 	}
 
+	if (criteria->pid) {
+		if (criteria->pid != view->pid) {
+			return false;
+		}
+	}
+
 	return true;
 }
 
@@ -377,23 +397,27 @@ struct match_data {
 	list_t *matches;
 };
 
-static void criteria_get_views_iterator(struct sway_container *container,
+static void criteria_get_containers_iterator(struct sway_container *container,
 		void *data) {
 	struct match_data *match_data = data;
 	if (container->view) {
 		if (criteria_matches_view(match_data->criteria, container->view)) {
-			list_add(match_data->matches, container->view);
+			list_add(match_data->matches, container);
+		}
+	} else if (has_container_criteria(match_data->criteria)) {
+		if (criteria_matches_container(match_data->criteria, container)) {
+			list_add(match_data->matches, container);
 		}
 	}
 }
 
-list_t *criteria_get_views(struct criteria *criteria) {
+list_t *criteria_get_containers(struct criteria *criteria) {
 	list_t *matches = create_list();
 	struct match_data data = {
 		.criteria = criteria,
 		.matches = matches,
 	};
-	root_for_each_container(criteria_get_views_iterator, &data);
+	root_for_each_container(criteria_get_containers_iterator, &data);
 	return matches;
 }
 
@@ -441,6 +465,7 @@ enum criteria_token {
 	T_TITLE,
 	T_URGENT,
 	T_WORKSPACE,
+	T_PID,
 
 	T_INVALID,
 };
@@ -476,6 +501,8 @@ static enum criteria_token token_from_name(char *name) {
 		return T_TILING;
 	} else if (strcmp(name, "floating") == 0) {
 		return T_FLOATING;
+	} else if (strcmp(name, "pid") == 0) {
+		return T_PID;
 	}
 	return T_INVALID;
 }
@@ -570,6 +597,12 @@ static bool parse_token(struct criteria *criteria, char *name, char *value) {
 	case T_WORKSPACE:
 		pattern_create(&criteria->workspace, value);
 		break;
+	case T_PID:
+		criteria->pid = strtoul(value, &endptr, 10);
+		if (*endptr != 0) {
+			error = strdup("The value for 'pid' should be numeric");
+		}
+		break;
 	case T_INVALID:
 		break;
 	}
