@@ -25,13 +25,16 @@ static void apply_exclusive(struct wlr_box *usable_area,
 		return;
 	}
 	struct {
-		uint32_t anchors;
+		uint32_t singular_anchor;
+		uint32_t anchor_triplet;
 		int *positive_axis;
 		int *negative_axis;
 		int margin;
 	} edges[] = {
+		// Top
 		{
-			.anchors =
+			.singular_anchor = ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP,
+			.anchor_triplet =
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP,
@@ -39,8 +42,10 @@ static void apply_exclusive(struct wlr_box *usable_area,
 			.negative_axis = &usable_area->height,
 			.margin = margin_top,
 		},
+		// Bottom
 		{
-			.anchors =
+			.singular_anchor = ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM,
+			.anchor_triplet =
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM,
@@ -48,8 +53,10 @@ static void apply_exclusive(struct wlr_box *usable_area,
 			.negative_axis = &usable_area->height,
 			.margin = margin_bottom,
 		},
+		// Left
 		{
-			.anchors =
+			.singular_anchor = ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT,
+			.anchor_triplet =
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM,
@@ -57,8 +64,10 @@ static void apply_exclusive(struct wlr_box *usable_area,
 			.negative_axis = &usable_area->width,
 			.margin = margin_left,
 		},
+		// Right
 		{
-			.anchors =
+			.singular_anchor = ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT,
+			.anchor_triplet =
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP |
 				ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM,
@@ -68,13 +77,15 @@ static void apply_exclusive(struct wlr_box *usable_area,
 		},
 	};
 	for (size_t i = 0; i < sizeof(edges) / sizeof(edges[0]); ++i) {
-		if ((anchor & edges[i].anchors) == edges[i].anchors && exclusive + edges[i].margin > 0) {
+		if ((anchor  == edges[i].singular_anchor || anchor == edges[i].anchor_triplet)
+				&& exclusive + edges[i].margin > 0) {
 			if (edges[i].positive_axis) {
 				*edges[i].positive_axis += exclusive + edges[i].margin;
 			}
 			if (edges[i].negative_axis) {
 				*edges[i].negative_axis -= exclusive + edges[i].margin;
 			}
+			break;
 		}
 	}
 }
@@ -222,7 +233,7 @@ void arrange_layers(struct sway_output *output) {
 	}
 }
 
-static struct sway_layer_surface *find_any_layer_by_client(
+static struct sway_layer_surface *find_mapped_layer_by_client(
 		struct wl_client *client, struct wlr_output *ignore_output) {
 	for (int i = 0; i < root->outputs->length; ++i) {
 		struct sway_output *output = root->outputs->items[i];
@@ -234,7 +245,8 @@ static struct sway_layer_surface *find_any_layer_by_client(
 		wl_list_for_each(lsurface,
 				&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY], link) {
 			struct wl_resource *resource = lsurface->layer_surface->resource;
-			if (wl_resource_get_client(resource) == client) {
+			if (wl_resource_get_client(resource) == client
+					&& lsurface->layer_surface->mapped) {
 				return lsurface;
 			}
 		}
@@ -258,7 +270,7 @@ static void handle_output_destroy(struct wl_listener *listener, void *data) {
 
 	if (set_focus) {
 		struct sway_layer_surface *layer =
-			find_any_layer_by_client(client, sway_layer->layer_surface->output);
+			find_mapped_layer_by_client(client, sway_layer->layer_surface->output);
 		if (layer) {
 			seat_set_focus_layer(seat, layer->layer_surface);
 		}
