@@ -171,8 +171,8 @@ void output_view_for_each_surface(struct sway_output *output,
 			- view->geometry.x,
 		.oy = view->container->surface_y - output->ly
 			- view->geometry.y,
-		.width = view->container->surface_width,
-		.height = view->container->surface_height,
+		.width = view->container->current.content_width,
+		.height = view->container->current.content_height,
 		.rotation = 0, // TODO
 	};
 
@@ -191,8 +191,8 @@ void output_view_for_each_popup(struct sway_output *output,
 			- view->geometry.x,
 		.oy = view->container->surface_y - output->ly
 			- view->geometry.y,
-		.width = view->container->surface_width,
-		.height = view->container->surface_height,
+		.width = view->container->current.content_width,
+		.height = view->container->current.content_height,
 		.rotation = 0, // TODO
 	};
 
@@ -243,6 +243,61 @@ void output_layer_for_each_surface(struct sway_output *output,
 	}
 }
 
+void output_layer_for_each_surface_toplevel(struct sway_output *output,
+		struct wl_list *layer_surfaces, sway_surface_iterator_func_t iterator,
+		void *user_data) {
+	struct sway_layer_surface *layer_surface;
+	wl_list_for_each(layer_surface, layer_surfaces, link) {
+		struct wlr_layer_surface_v1 *wlr_layer_surface_v1 =
+			layer_surface->layer_surface;
+		output_surface_for_each_surface(output, wlr_layer_surface_v1->surface,
+			layer_surface->geo.x, layer_surface->geo.y, iterator,
+			user_data);
+	}
+}
+
+
+void output_layer_for_each_surface_popup(struct sway_output *output,
+		struct wl_list *layer_surfaces, sway_surface_iterator_func_t iterator,
+		void *user_data) {
+	struct sway_layer_surface *layer_surface;
+	wl_list_for_each(layer_surface, layer_surfaces, link) {
+		struct wlr_layer_surface_v1 *wlr_layer_surface_v1 =
+			layer_surface->layer_surface;
+
+		struct wlr_xdg_popup *state;
+		wl_list_for_each(state, &wlr_layer_surface_v1->popups, link) {
+			struct wlr_xdg_surface *popup = state->base;
+			if (!popup->configured) {
+				continue;
+			}
+
+			double popup_sx, popup_sy;
+			popup_sx = layer_surface->geo.x +
+				popup->popup->geometry.x - popup->geometry.x;
+			popup_sy = layer_surface->geo.y +
+				popup->popup->geometry.y - popup->geometry.y;
+
+			struct wlr_surface *surface = popup->surface;
+
+			struct surface_iterator_data data = {
+				.user_iterator = iterator,
+				.user_data = user_data,
+				.output = output,
+				.view = NULL,
+				.ox = popup_sx,
+				.oy = popup_sy,
+				.width = surface->current.width,
+				.height = surface->current.height,
+				.rotation = 0,
+			};
+
+			wlr_xdg_surface_for_each_surface(
+					popup, output_for_each_surface_iterator, &data);
+		}
+	}
+}
+
 #if HAVE_XWAYLAND
 void output_unmanaged_for_each_surface(struct sway_output *output,
 		struct wl_list *unmanaged, sway_surface_iterator_func_t iterator,
@@ -379,15 +434,10 @@ struct sway_workspace *output_get_active_workspace(struct sway_output *output) {
 }
 
 bool output_has_opaque_overlay_layer_surface(struct sway_output *output) {
-	struct wlr_layer_surface_v1 *wlr_layer_surface_v1;
-	wl_list_for_each(wlr_layer_surface_v1, &server.layer_shell->surfaces, link) {
-		if (wlr_layer_surface_v1->output != output->wlr_output ||
-				wlr_layer_surface_v1->current.layer != ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY) {
-			continue;
-		}
-		struct wlr_surface *wlr_surface = wlr_layer_surface_v1->surface;
-		struct sway_layer_surface *sway_layer_surface =
-			layer_from_wlr_layer_surface_v1(wlr_layer_surface_v1);
+	struct sway_layer_surface *sway_layer_surface;
+	wl_list_for_each(sway_layer_surface,
+			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY], link) {
+		struct wlr_surface *wlr_surface = sway_layer_surface->layer_surface->surface;
 		pixman_box32_t output_box = {
 			.x2 = output->width,
 			.y2 = output->height,
@@ -453,7 +503,7 @@ static bool scan_out_fullscreen_view(struct sway_output *output,
 		return false;
 	}
 
-	if (view->saved_buffer) {
+	if (!wl_list_empty(&view->saved_buffers)) {
 		return false;
 	}
 
@@ -501,13 +551,11 @@ static bool scan_out_fullscreen_view(struct sway_output *output,
 	wlr_presentation_surface_sampled_on_output(server.presentation, surface,
 		wlr_output);
 
-	if (!wlr_output_attach_buffer(wlr_output, surface->buffer)) {
-		return false;
-	}
+	wlr_output_attach_buffer(wlr_output, &surface->buffer->base);
 	return wlr_output_commit(wlr_output);
 }
 
-int output_repaint_timer_handler(void *data) {
+static int output_repaint_timer_handler(void *data) {
 	struct sway_output *output = data;
 	if (output->wlr_output == NULL) {
 		return 0;
@@ -515,9 +563,6 @@ int output_repaint_timer_handler(void *data) {
 
 	output->wlr_output->frame_pending = false;
 
-	bool surface_needs_frame = output->surface_needs_frame;
-	output->surface_needs_frame = false;
-
 	struct sway_workspace *workspace = output->current.active_workspace;
 	if (workspace == NULL) {
 		return 0;
@@ -562,10 +607,6 @@ int output_repaint_timer_handler(void *data) {
 		output_render(output, &now, &damage);
 	} else {
 		wlr_output_rollback(output->wlr_output);
-
-		if (surface_needs_frame) {
-			wlr_output_schedule_frame(output->wlr_output);
-		}
 	}
 
 	pixman_region32_fini(&damage);
@@ -681,8 +722,9 @@ static void damage_surface_iterator(struct sway_output *output, struct sway_view
 		wlr_output_damage_add_box(output->damage, &box);
 	}
 
-	output->surface_needs_frame = true;
-	wlr_output_schedule_frame(output->wlr_output);
+	if (!wl_list_empty(&surface->current.frame_callback_list)) {
+		wlr_output_schedule_frame(output->wlr_output);
+	}
 }
 
 void output_damage_surface(struct sway_output *output, double ox, double oy,
@@ -747,6 +789,10 @@ static void damage_handle_destroy(struct wl_listener *listener, void *data) {
 		return;
 	}
 	output_disable(output);
+
+	wl_list_remove(&output->damage_destroy.link);
+	wl_list_remove(&output->damage_frame.link);
+
 	transaction_commit_dirty();
 }
 
@@ -790,8 +836,6 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 	wl_list_remove(&output->transform.link);
 	wl_list_remove(&output->scale.link);
 	wl_list_remove(&output->present.link);
-	wl_list_remove(&output->damage_destroy.link);
-	wl_list_remove(&output->damage_frame.link);
 
 	transaction_commit_dirty();
 
@@ -800,7 +844,7 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 
 static void handle_mode(struct wl_listener *listener, void *data) {
 	struct sway_output *output = wl_container_of(listener, output, mode);
-	if (!output->configured && !output->enabled) {
+	if (!output->enabled && !output->enabling) {
 		struct output_config *oc = find_output_config(output);
 		if (output->wlr_output->current_mode != NULL &&
 				(!oc || oc->enabled)) {
@@ -813,7 +857,7 @@ static void handle_mode(struct wl_listener *listener, void *data) {
 		}
 		return;
 	}
-	if (!output->enabled || !output->configured) {
+	if (!output->enabled) {
 		return;
 	}
 	arrange_layers(output);
@@ -825,7 +869,7 @@ static void handle_mode(struct wl_listener *listener, void *data) {
 
 static void handle_transform(struct wl_listener *listener, void *data) {
 	struct sway_output *output = wl_container_of(listener, output, transform);
-	if (!output->enabled || !output->configured) {
+	if (!output->enabled) {
 		return;
 	}
 	arrange_layers(output);
@@ -842,7 +886,7 @@ static void update_textures(struct sway_container *con, void *data) {
 
 static void handle_scale(struct wl_listener *listener, void *data) {
 	struct sway_output *output = wl_container_of(listener, output, scale);
-	if (!output->enabled || !output->configured) {
+	if (!output->enabled) {
 		return;
 	}
 	arrange_layers(output);
@@ -896,11 +940,7 @@ void handle_new_output(struct wl_listener *listener, void *data) {
 		output_repaint_timer_handler, output);
 
 	struct output_config *oc = find_output_config(output);
-	if (!oc || oc->enabled) {
-		output_enable(output, oc);
-	} else {
-		wlr_output_enable(output->wlr_output, false);
-	}
+	apply_output_config(oc, output);
 	free_output_config(oc);
 
 	transaction_commit_dirty();
@@ -915,10 +955,9 @@ void handle_output_layout_change(struct wl_listener *listener,
 	update_output_manager_config(server);
 }
 
-void handle_output_manager_apply(struct wl_listener *listener, void *data) {
-	struct sway_server *server =
-		wl_container_of(listener, server, output_manager_apply);
-	struct wlr_output_configuration_v1 *config = data;
+static void output_manager_apply(struct sway_server *server,
+		struct wlr_output_configuration_v1 *config, bool test_only) {
+	// TODO: perform atomic tests on the whole backend atomically
 
 	struct wlr_output_configuration_head_v1 *config_head;
 	// First disable outputs we need to disable
@@ -932,8 +971,12 @@ void handle_output_manager_apply(struct wl_listener *listener, void *data) {
 		struct output_config *oc = new_output_config(output->wlr_output->name);
 		oc->enabled = false;
 
-		oc = store_output_config(oc);
-		ok &= apply_output_config(oc, output);
+		if (test_only) {
+			ok &= test_output_config(oc, output);
+		} else {
+			oc = store_output_config(oc);
+			ok &= apply_output_config(oc, output);
+		}
 	}
 
 	// Then enable outputs that need to
@@ -960,8 +1003,12 @@ void handle_output_manager_apply(struct wl_listener *listener, void *data) {
 		oc->transform = config_head->state.transform;
 		oc->scale = config_head->state.scale;
 
-		oc = store_output_config(oc);
-		ok &= apply_output_config(oc, output);
+		if (test_only) {
+			ok &= test_output_config(oc, output);
+		} else {
+			oc = store_output_config(oc);
+			ok &= apply_output_config(oc, output);
+		}
 	}
 
 	if (ok) {
@@ -971,13 +1018,41 @@ void handle_output_manager_apply(struct wl_listener *listener, void *data) {
 	}
 	wlr_output_configuration_v1_destroy(config);
 
-	update_output_manager_config(server);
+	if (!test_only) {
+		update_output_manager_config(server);
+	}
+}
+
+void handle_output_manager_apply(struct wl_listener *listener, void *data) {
+	struct sway_server *server =
+		wl_container_of(listener, server, output_manager_apply);
+	struct wlr_output_configuration_v1 *config = data;
+
+	output_manager_apply(server, config, false);
 }
 
 void handle_output_manager_test(struct wl_listener *listener, void *data) {
+	struct sway_server *server =
+		wl_container_of(listener, server, output_manager_test);
 	struct wlr_output_configuration_v1 *config = data;
 
-	// TODO: implement test-only mode
-	wlr_output_configuration_v1_send_succeeded(config);
-	wlr_output_configuration_v1_destroy(config);
+	output_manager_apply(server, config, true);
+}
+
+void handle_output_power_manager_set_mode(struct wl_listener *listener,
+		void *data) {
+	struct wlr_output_power_v1_set_mode_event *event = data;
+	struct sway_output *output = event->output->data;
+
+	struct output_config *oc = new_output_config(output->wlr_output->name);
+	switch (event->mode) {
+	case ZWLR_OUTPUT_POWER_V1_MODE_OFF:
+		oc->dpms_state = DPMS_OFF;
+		break;
+	case ZWLR_OUTPUT_POWER_V1_MODE_ON:
+		oc->dpms_state = DPMS_ON;
+		break;
+	}
+	oc = store_output_config(oc);
+	apply_output_config(oc, output);
 }
