@@ -94,6 +94,22 @@ static void unmanaged_handle_unmap(struct wl_listener *listener, void *data) {
 	struct sway_seat *seat = input_manager_current_seat();
 	if (seat->wlr_seat->keyboard_state.focused_surface ==
 			xsurface->surface) {
+
+		// Try to find another unmanaged surface from the same process to pass
+		// focus to. This is necessary because some applications (e.g. Jetbrains
+		// IDEs) represent their multi-level menus as unmanaged surfaces, and
+		// when closing a submenu, the main menu should get input focus.
+		struct sway_xwayland_unmanaged *current;
+		wl_list_for_each(current, &root->xwayland_unmanaged, link) {
+			struct wlr_xwayland_surface *prev_xsurface =
+				current->wlr_xwayland_surface;
+			if (prev_xsurface->pid == xsurface->pid &&
+					wlr_xwayland_or_surface_wants_focus(prev_xsurface)) {
+				seat_set_focus_surface(seat, prev_xsurface->surface, false);
+				return;
+			}
+		}
+
 		// Restore focus
 		struct sway_node *previous = seat_get_focus_inactive(seat, &root->node);
 		if (previous) {
@@ -178,7 +194,10 @@ static uint32_t get_int_prop(struct sway_view *view, enum sway_view_prop prop) {
 		}
 		return 0;
 	case VIEW_PROP_WINDOW_TYPE:
-		return *view->wlr_xwayland_surface->window_type;
+		if (view->wlr_xwayland_surface->window_type_len == 0) {
+			return 0;
+		}
+		return view->wlr_xwayland_surface->window_type[0];
 	default:
 		return 0;
 	}
@@ -352,10 +371,11 @@ static void handle_commit(struct wl_listener *listener, void *data) {
 	} else {
 		struct wlr_box new_geo;
 		get_geometry(view, &new_geo);
-		struct sway_container *con = view->container;
 
-		if ((new_geo.width != con->surface_width ||
-					new_geo.height != con->surface_height)) {
+		if ((new_geo.width != view->geometry.width ||
+					new_geo.height != view->geometry.height ||
+					new_geo.x != view->geometry.x ||
+					new_geo.y != view->geometry.y)) {
 			// The view has unexpectedly sent a new size
 			// eg. The Firefox "Save As" dialog when downloading a file
 			desktop_damage_view(view);
@@ -363,6 +383,8 @@ static void handle_commit(struct wl_listener *listener, void *data) {
 			memcpy(&view->geometry, &new_geo, sizeof(struct wlr_box));
 			desktop_damage_view(view);
 			transaction_commit_dirty();
+			transaction_notify_view_ready_by_size(view,
+					new_geo.width, new_geo.height);
 		} else {
 			memcpy(&view->geometry, &new_geo, sizeof(struct wlr_box));
 		}
@@ -584,7 +606,7 @@ static void handle_set_hints(struct wl_listener *listener, void *data) {
 		return;
 	}
 	if (!xsurface->hints_urgency && view->urgent_timer) {
-		// The view is is in the timeout period. We'll ignore the request to
+		// The view is in the timeout period. We'll ignore the request to
 		// unset urgency so that the view remains urgent until the timer clears
 		// it.
 		return;
