@@ -4,15 +4,20 @@
 #include <string.h>
 #include <math.h>
 #include <wlr/backend/libinput.h>
+#include <wlr/types/wlr_cursor.h>
+#include <wlr/types/wlr_keyboard_group.h>
 #include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_virtual_keyboard_v1.h>
 #include <wlr/types/wlr_virtual_pointer_v1.h>
 #include "sway/config.h"
+#include "sway/input/cursor.h"
 #include "sway/input/input-manager.h"
+#include "sway/input/keyboard.h"
 #include "sway/input/libinput.h"
 #include "sway/input/seat.h"
 #include "sway/ipc-server.h"
 #include "sway/server.h"
+#include "sway/tree/view.h"
 #include "stringop.h"
 #include "list.h"
 #include "log.h"
@@ -65,7 +70,8 @@ char *input_device_get_identifier(struct wlr_input_device *device) {
 
 	char *p = name;
 	for (; *p; ++p) {
-		if (*p == ' ') {
+		// There are in fact input devices with unprintable characters in its name
+		if (*p == ' ' || !isprint(*p)) {
 			*p = '_';
 		}
 	}
@@ -294,6 +300,79 @@ static void handle_inhibit_deactivate(struct wl_listener *listener, void *data)
 	}
 }
 
+static void handle_keyboard_shortcuts_inhibitor_destroy(
+		struct wl_listener *listener, void *data) {
+	struct sway_keyboard_shortcuts_inhibitor *sway_inhibitor =
+		wl_container_of(listener, sway_inhibitor, destroy);
+
+	sway_log(SWAY_DEBUG, "Removing keyboard shortcuts inhibitor");
+
+	// sway_seat::keyboard_shortcuts_inhibitors
+	wl_list_remove(&sway_inhibitor->link);
+	wl_list_remove(&sway_inhibitor->destroy.link);
+	free(sway_inhibitor);
+}
+
+static void handle_keyboard_shortcuts_inhibit_new_inhibitor(
+		struct wl_listener *listener, void *data) {
+	struct sway_input_manager *input_manager =
+		wl_container_of(listener, input_manager,
+				keyboard_shortcuts_inhibit_new_inhibitor);
+	struct wlr_keyboard_shortcuts_inhibitor_v1 *inhibitor = data;
+
+	sway_log(SWAY_DEBUG, "Adding keyboard shortcuts inhibitor");
+
+	struct sway_keyboard_shortcuts_inhibitor *sway_inhibitor =
+		calloc(1, sizeof(struct sway_keyboard_shortcuts_inhibitor));
+	if (!sway_assert(sway_inhibitor, "could not allocate keyboard "
+				"shortcuts inhibitor")) {
+		return;
+	}
+	sway_inhibitor->inhibitor = inhibitor;
+
+	sway_inhibitor->destroy.notify = handle_keyboard_shortcuts_inhibitor_destroy;
+	wl_signal_add(&inhibitor->events.destroy, &sway_inhibitor->destroy);
+
+	// attach inhibitor to the seat it applies to
+	struct sway_seat *seat = inhibitor->seat->data;
+	wl_list_insert(&seat->keyboard_shortcuts_inhibitors, &sway_inhibitor->link);
+
+	// per-view, seat-agnostic config via criteria
+	struct sway_view *view = view_from_wlr_surface(inhibitor->surface);
+	enum seat_config_shortcuts_inhibit inhibit = SHORTCUTS_INHIBIT_DEFAULT;
+	if (view) {
+		inhibit = view->shortcuts_inhibit;
+	}
+
+	if (inhibit == SHORTCUTS_INHIBIT_DEFAULT) {
+		struct seat_config *config = seat_get_config(seat);
+		if (!config) {
+			config = seat_get_config_by_name("*");
+		}
+
+		if (config) {
+			inhibit = config->shortcuts_inhibit;
+		}
+	}
+
+	if (inhibit == SHORTCUTS_INHIBIT_DISABLE) {
+		/**
+		 * Here we deny to honour the inhibitor by never sending the
+		 * activate signal. We can not, however, destroy the inhibitor
+		 * because the protocol doesn't allow for it. So it will linger
+		 * until the client removes it im- or explicitly. But at least
+		 * it can only be one inhibitor per surface and seat at a time.
+		 *
+		 * We also want to allow the user to activate the inhibitor
+		 * manually later which is why we do this check here where the
+		 * inhibitor is already attached to its seat and ready for use.
+		 */
+		return;
+	}
+
+	wlr_keyboard_shortcuts_inhibitor_v1_activate(inhibitor);
+}
+
 void handle_virtual_keyboard(struct wl_listener *listener, void *data) {
 	struct sway_input_manager *input_manager =
 		wl_container_of(listener, input_manager, virtual_keyboard_new);
@@ -312,6 +391,7 @@ void handle_virtual_keyboard(struct wl_listener *listener, void *data) {
 	}
 	device->data = input_device;
 
+	input_device->is_virtual = true;
 	input_device->wlr_device = device;
 	input_device->identifier = input_device_get_identifier(device);
 	wl_list_insert(&input_manager->devices, &input_device->link);
@@ -343,6 +423,7 @@ void handle_virtual_pointer(struct wl_listener *listener, void *data) {
 	}
 	device->data = input_device;
 
+	input_device->is_virtual = true;
 	input_device->wlr_device = device;
 	input_device->identifier = input_device_get_identifier(device);
 	wl_list_insert(&input_manager->devices, &input_device->link);
@@ -354,6 +435,11 @@ void handle_virtual_pointer(struct wl_listener *listener, void *data) {
 	input_device->device_destroy.notify = handle_device_destroy;
 
 	seat_add_device(seat, input_device);
+
+	if (event->suggested_output) {
+		wlr_cursor_map_input_to_output(seat->cursor->cursor, device,
+			event->suggested_output);
+	}
 }
 
 struct sway_input_manager *input_manager_create(struct sway_server *server) {
@@ -390,6 +476,13 @@ struct sway_input_manager *input_manager_create(struct sway_server *server) {
 	wl_signal_add(&input->inhibit->events.deactivate,
 			&input->inhibit_deactivate);
 
+	input->keyboard_shortcuts_inhibit =
+		wlr_keyboard_shortcuts_inhibit_v1_create(server->wl_display);
+	input->keyboard_shortcuts_inhibit_new_inhibitor.notify =
+		handle_keyboard_shortcuts_inhibit_new_inhibitor;
+	wl_signal_add(&input->keyboard_shortcuts_inhibit->events.new_inhibitor,
+			&input->keyboard_shortcuts_inhibit_new_inhibitor);
+
 	return input;
 }
 
@@ -429,6 +522,22 @@ static void retranslate_keysyms(struct input_config *input_config) {
 	}
 }
 
+static void input_manager_configure_input(
+		struct sway_input_device *input_device) {
+	sway_input_configure_libinput_device(input_device);
+	struct sway_seat *seat = NULL;
+	wl_list_for_each(seat, &server.input->seats, link) {
+		seat_configure_device(seat, input_device);
+	}
+}
+
+void input_manager_configure_all_inputs(void) {
+	struct sway_input_device *input_device = NULL;
+	wl_list_for_each(input_device, &server.input->devices, link) {
+		input_manager_configure_input(input_device);
+	}
+}
+
 void input_manager_apply_input_config(struct input_config *input_config) {
 	struct sway_input_device *input_device = NULL;
 	bool wildcard = strcmp(input_config->identifier, "*") == 0;
@@ -439,11 +548,7 @@ void input_manager_apply_input_config(struct input_config *input_config) {
 		if (strcmp(input_device->identifier, input_config->identifier) == 0
 				|| wildcard
 				|| type_matches) {
-			sway_input_configure_libinput_device(input_device);
-			struct sway_seat *seat = NULL;
-			wl_list_for_each(seat, &server.input->seats, link) {
-				seat_configure_device(seat, input_device);
-			}
+			input_manager_configure_input(input_device);
 		}
 	}
 
@@ -458,13 +563,23 @@ void input_manager_reset_input(struct sway_input_device *input_device) {
 	}
 }
 
-void input_manager_reset_all_inputs() {
+void input_manager_reset_all_inputs(void) {
 	struct sway_input_device *input_device = NULL;
 	wl_list_for_each(input_device, &server.input->devices, link) {
 		input_manager_reset_input(input_device);
 	}
-}
 
+	// If there is at least one keyboard using the default keymap, repeat delay,
+	// and repeat rate, then it is possible that there is a keyboard group that
+	// need their keyboard disarmed.
+	struct sway_seat *seat;
+	wl_list_for_each(seat, &server.input->seats, link) {
+		struct sway_keyboard_group *group;
+		wl_list_for_each(group, &seat->keyboard_groups, link) {
+			sway_keyboard_disarm_key_repeat(group->seat_device->keyboard);
+		}
+	}
+}
 
 void input_manager_apply_seat_config(struct seat_config *seat_config) {
 	sway_log(SWAY_DEBUG, "applying seat config for seat %s", seat_config->name);
