@@ -7,6 +7,7 @@
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_data_device.h>
 #include <wlr/types/wlr_idle.h>
+#include <wlr/types/wlr_keyboard_group.h>
 #include <wlr/types/wlr_output_layout.h>
 #include <wlr/types/wlr_primary_selection.h>
 #include <wlr/types/wlr_tablet_v2.h>
@@ -179,6 +180,7 @@ static void seat_send_focus(struct sway_node *node, struct sway_seat *seat) {
 
 		seat_keyboard_notify_enter(seat, view->surface);
 		seat_tablet_pads_notify_enter(seat, view->surface);
+		sway_input_method_relay_set_focus(&seat->im_relay, view->surface);
 
 		struct wlr_pointer_constraint_v1 *constraint =
 			wlr_pointer_constraints_v1_constraint_for_surface(
@@ -288,8 +290,11 @@ static void handle_seat_node_destroy(struct wl_listener *listener, void *data) {
 		}
 		// The structure change might have caused it to move up to the top of
 		// the focus stack without sending focus notifications to the view
-		seat_send_focus(next_focus, seat);
-		seat_set_focus(seat, next_focus);
+		if (seat_get_focus(seat) == next_focus) {
+			seat_send_focus(next_focus, seat);
+		} else {
+			seat_set_focus(seat, next_focus);
+		}
 	} else {
 		// Setting focus_inactive
 		focus = seat_get_focus_inactive(seat, &root->node);
@@ -397,6 +402,31 @@ static void drag_icon_handle_destroy(struct wl_listener *listener, void *data) {
 	free(icon);
 }
 
+static void drag_handle_destroy(struct wl_listener *listener, void *data) {
+	struct sway_drag *drag = wl_container_of(listener, drag, destroy);
+
+	// Focus enter isn't sent during drag, so refocus the focused node, layer
+	// surface or unmanaged surface.
+	struct sway_seat *seat = drag->seat;
+	struct sway_node *focus = seat_get_focus(seat);
+	if (focus) {
+		seat_set_focus(seat, NULL);
+		seat_set_focus(seat, focus);
+	} else if (seat->focused_layer) {
+		struct wlr_layer_surface_v1 *layer = seat->focused_layer;
+		seat_set_focus_layer(seat, NULL);
+		seat_set_focus_layer(seat, layer);
+	} else {
+		struct wlr_surface *unmanaged = seat->wlr_seat->keyboard_state.focused_surface;
+		seat_set_focus_surface(seat, NULL, false);
+		seat_set_focus_surface(seat, unmanaged, false);
+	}
+
+	drag->wlr_drag->data = NULL;
+	wl_list_remove(&drag->destroy.link);
+	free(drag);
+}
+
 static void handle_request_start_drag(struct wl_listener *listener,
 		void *data) {
 	struct sway_seat *seat = wl_container_of(listener, seat, request_start_drag);
@@ -426,6 +456,19 @@ static void handle_request_start_drag(struct wl_listener *listener,
 static void handle_start_drag(struct wl_listener *listener, void *data) {
 	struct sway_seat *seat = wl_container_of(listener, seat, start_drag);
 	struct wlr_drag *wlr_drag = data;
+
+	struct sway_drag *drag = calloc(1, sizeof(struct sway_drag));
+	if (drag == NULL) {
+		sway_log(SWAY_ERROR, "Allocation failed");
+		return;
+	}
+	drag->seat = seat;
+	drag->wlr_drag = wlr_drag;
+	wlr_drag->data = drag;
+
+	drag->destroy.notify = drag_handle_destroy;
+	wl_signal_add(&wlr_drag->events.destroy, &drag->destroy);
+
 	struct wlr_drag_icon *wlr_drag_icon = wlr_drag->icon;
 	if (wlr_drag_icon == NULL) {
 		return;
@@ -529,14 +572,6 @@ struct sway_seat *seat_create(const char *seat_name) {
 
 	seat->deferred_bindings = create_list();
 
-	if (!wl_list_empty(&server.input->seats)) {
-		// Since this is not the first seat, attempt to set initial focus
-		struct sway_seat *current_seat = input_manager_current_seat();
-		struct sway_node *current_focus =
-			seat_get_focus_inactive(current_seat, &root->node);
-		seat_set_focus(seat, current_focus);
-	}
-
 	wl_signal_add(&root->events.new_node, &seat->new_node);
 	seat->new_node.notify = handle_new_node;
 
@@ -557,9 +592,21 @@ struct sway_seat *seat_create(const char *seat_name) {
 		handle_request_set_primary_selection;
 
 	wl_list_init(&seat->keyboard_groups);
+	wl_list_init(&seat->keyboard_shortcuts_inhibitors);
 
+	sway_input_method_relay_init(seat, &seat->im_relay);
+
+	bool first = wl_list_empty(&server.input->seats);
 	wl_list_insert(&server.input->seats, &seat->link);
 
+	if (!first) {
+		// Since this is not the first seat, attempt to set initial focus
+		struct sway_seat *current_seat = input_manager_current_seat();
+		struct sway_node *current_focus =
+			seat_get_focus_inactive(current_seat, &root->node);
+		seat_set_focus(seat, current_focus);
+	}
+
 	seatop_begin_default(seat);
 
 	return seat;
@@ -645,6 +692,8 @@ static void seat_apply_input_config(struct sway_seat *seat,
 		}
 		struct sway_output *output = output_by_name_or_id(mapped_to_output);
 		if (!output) {
+			sway_log(SWAY_DEBUG, "Requested output %s for device %s isn't present",
+				mapped_to_output, sway_device->input_device->identifier);
 			return;
 		}
 		wlr_cursor_map_input_to_output(seat->cursor->cursor,
@@ -688,13 +737,13 @@ static void seat_configure_keyboard(struct sway_seat *seat,
 	struct sway_node *focus = seat_get_focus(seat);
 	if (focus && node_is_view(focus)) {
 		// force notify reenter to pick up the new configuration
-		wlr_seat_keyboard_clear_focus(seat->wlr_seat);
+		wlr_seat_keyboard_notify_clear_focus(seat->wlr_seat);
 		seat_keyboard_notify_enter(seat, focus->sway_container->view->surface);
 	}
 }
 
 static void seat_configure_switch(struct sway_seat *seat,
-        struct sway_seat_device *seat_device) {
+		struct sway_seat_device *seat_device) {
 	if (!seat_device->switch_device) {
 		sway_switch_create(seat, seat_device);
 	}
@@ -722,7 +771,7 @@ static void seat_configure_tablet_tool(struct sway_seat *seat,
 
 static void seat_configure_tablet_pad(struct sway_seat *seat,
 		struct sway_seat_device *sway_device) {
-	if (!sway_device->tablet) {
+	if (!sway_device->tablet_pad) {
 		sway_device->tablet_pad = sway_tablet_pad_create(seat, sway_device);
 	}
 	sway_configure_tablet_pad(sway_device->tablet_pad);
@@ -737,6 +786,13 @@ static struct sway_seat_device *seat_get_device(struct sway_seat *seat,
 		}
 	}
 
+	struct sway_keyboard_group *group = NULL;
+	wl_list_for_each(group, &seat->keyboard_groups, link) {
+		if (group->seat_device->input_device == input_device) {
+			return group->seat_device;
+		}
+	}
+
 	return NULL;
 }
 
@@ -869,7 +925,7 @@ void seat_configure_xcursor(struct sway_seat *seat) {
 		}
 
 #if HAVE_XWAYLAND
-		if (config->xwayland && (!server.xwayland.xcursor_manager ||
+		if (server.xwayland.wlr_xwayland && (!server.xwayland.xcursor_manager ||
 				!xcursor_manager_is_named(server.xwayland.xcursor_manager,
 					cursor_theme) ||
 				server.xwayland.xcursor_manager->size != cursor_size)) {
@@ -905,8 +961,10 @@ void seat_configure_xcursor(struct sway_seat *seat) {
 		wlr_xcursor_manager_destroy(seat->cursor->xcursor_manager);
 		seat->cursor->xcursor_manager =
 			wlr_xcursor_manager_create(cursor_theme, cursor_size);
-		sway_assert(seat->cursor->xcursor_manager,
-					"Cannot create XCursor manager for theme");
+		if (!seat->cursor->xcursor_manager) {
+			sway_log(SWAY_ERROR,
+				"Cannot create XCursor manager for theme '%s'", cursor_theme);
+		}
 	}
 
 	for (int i = 0; i < root->outputs->length; ++i) {
@@ -915,13 +973,15 @@ void seat_configure_xcursor(struct sway_seat *seat) {
 		bool result =
 			wlr_xcursor_manager_load(seat->cursor->xcursor_manager,
 				output->scale);
-
-		sway_assert(!result,
-			"Cannot load xcursor theme for output '%s' with scale %f",
-			// TODO: Fractional scaling
-			output->name, (double)output->scale);
+		if (!result) {
+			sway_log(SWAY_ERROR,
+				"Cannot load xcursor theme for output '%s' with scale %f",
+				output->name, output->scale);
+		}
 	}
 
+	// Reset the cursor so that we apply it to outputs that just appeared
+	cursor_set_image(seat->cursor, NULL, NULL);
 	cursor_set_image(seat->cursor, "left_ptr", NULL);
 	wlr_cursor_warp(seat->cursor->cursor, NULL, seat->cursor->cursor->x,
 		seat->cursor->cursor->y);
@@ -942,7 +1002,7 @@ static void send_unfocus(struct sway_container *con, void *data) {
 // Unfocus the container and any children (eg. when leaving `focus parent`)
 static void seat_send_unfocus(struct sway_node *node, struct sway_seat *seat) {
 	sway_cursor_constrain(seat->cursor, NULL);
-	wlr_seat_keyboard_clear_focus(seat->wlr_seat);
+	wlr_seat_keyboard_notify_clear_focus(seat->wlr_seat);
 	if (node->type == N_WORKSPACE) {
 		workspace_for_each_container(node->sway_workspace, send_unfocus, seat);
 	} else {
@@ -1011,6 +1071,7 @@ void seat_set_focus(struct sway_seat *seat, struct sway_node *node) {
 			view_close_popups(last_focus->sway_container->view);
 		}
 		seat_send_unfocus(last_focus, seat);
+		sway_input_method_relay_set_focus(&seat->im_relay, NULL);
 		seat->has_focus = false;
 		return;
 	}
@@ -1157,7 +1218,13 @@ void seat_set_focus_surface(struct sway_seat *seat,
 		seat_send_unfocus(focus, seat);
 		seat->has_focus = false;
 	}
-	seat_keyboard_notify_enter(seat, surface);
+
+	if (surface) {
+		seat_keyboard_notify_enter(seat, surface);
+	} else {
+		wlr_seat_keyboard_notify_clear_focus(seat->wlr_seat);
+	}
+
 	seat_tablet_pads_notify_enter(seat, surface);
 }
 
@@ -1208,7 +1275,7 @@ void seat_set_exclusive_client(struct sway_seat *seat,
 	}
 	if (seat->wlr_seat->pointer_state.focused_client) {
 		if (seat->wlr_seat->pointer_state.focused_client->client != client) {
-			wlr_seat_pointer_clear_focus(seat->wlr_seat);
+			wlr_seat_pointer_notify_clear_focus(seat->wlr_seat);
 		}
 	}
 	struct timespec now;
@@ -1358,9 +1425,9 @@ void seat_apply_config(struct sway_seat *seat,
 
 	wl_list_for_each(seat_device, &seat->devices, link) {
 		seat_configure_device(seat, seat_device->input_device);
+		cursor_handle_activity(seat->cursor,
+			seat_device->input_device->wlr_device);
 	}
-
-	cursor_handle_activity(seat->cursor);
 }
 
 struct seat_config *seat_get_config(struct sway_seat *seat) {
@@ -1435,16 +1502,34 @@ void seatop_button(struct sway_seat *seat, uint32_t time_msec,
 	}
 }
 
-void seatop_motion(struct sway_seat *seat, uint32_t time_msec,
+void seatop_pointer_motion(struct sway_seat *seat, uint32_t time_msec,
 		double dx, double dy) {
-	if (seat->seatop_impl->motion) {
-		seat->seatop_impl->motion(seat, time_msec, dx, dy);
+	if (seat->seatop_impl->pointer_motion) {
+		seat->seatop_impl->pointer_motion(seat, time_msec, dx, dy);
 	}
 }
 
-void seatop_axis(struct sway_seat *seat, struct wlr_event_pointer_axis *event) {
-	if (seat->seatop_impl->axis) {
-		seat->seatop_impl->axis(seat, event);
+void seatop_pointer_axis(struct sway_seat *seat,
+		struct wlr_event_pointer_axis *event) {
+	if (seat->seatop_impl->pointer_axis) {
+		seat->seatop_impl->pointer_axis(seat, event);
+	}
+}
+
+void seatop_tablet_tool_tip(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec,
+		enum wlr_tablet_tool_tip_state state) {
+	if (seat->seatop_impl->tablet_tool_tip) {
+		seat->seatop_impl->tablet_tool_tip(seat, tool, time_msec, state);
+	}
+}
+
+void seatop_tablet_tool_motion(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec, double dx, double dy) {
+	if (seat->seatop_impl->tablet_tool_motion) {
+		seat->seatop_impl->tablet_tool_motion(seat, tool, time_msec, dx, dy);
+	} else {
+		seatop_pointer_motion(seat, time_msec, dx, dy);
 	}
 }
 
@@ -1473,3 +1558,24 @@ void seatop_render(struct sway_seat *seat, struct sway_output *output,
 bool seatop_allows_set_cursor(struct sway_seat *seat) {
 	return seat->seatop_impl->allow_set_cursor;
 }
+
+struct sway_keyboard_shortcuts_inhibitor *
+keyboard_shortcuts_inhibitor_get_for_surface(
+		const struct sway_seat *seat,
+		const struct wlr_surface *surface) {
+	struct sway_keyboard_shortcuts_inhibitor *sway_inhibitor = NULL;
+	wl_list_for_each(sway_inhibitor, &seat->keyboard_shortcuts_inhibitors, link) {
+		if (sway_inhibitor->inhibitor->surface == surface) {
+			return sway_inhibitor;
+		}
+	}
+
+	return NULL;
+}
+
+struct sway_keyboard_shortcuts_inhibitor *
+keyboard_shortcuts_inhibitor_get_for_focused_surface(
+		const struct sway_seat *seat) {
+	return keyboard_shortcuts_inhibitor_get_for_surface(seat,
+		seat->wlr_seat->keyboard_state.focused_surface);
+}
