@@ -1,6 +1,7 @@
 #define _POSIX_C_SOURCE 200809L
 #include <float.h>
 #include <wlr/types/wlr_cursor.h>
+#include <wlr/types/wlr_tablet_v2.h>
 #include "sway/input/cursor.h"
 #include "sway/input/seat.h"
 #include "sway/tree/view.h"
@@ -12,7 +13,7 @@ struct seatop_down_event {
 	double ref_con_lx, ref_con_ly; // container's x/y at start of op
 };
 
-static void handle_axis(struct sway_seat *seat,
+static void handle_pointer_axis(struct sway_seat *seat,
 		struct wlr_event_pointer_axis *event) {
 	struct sway_input_device *input_device =
 		event->device ? event->device->data : NULL;
@@ -36,7 +37,7 @@ static void handle_button(struct sway_seat *seat, uint32_t time_msec,
 	}
 }
 
-static void handle_motion(struct sway_seat *seat, uint32_t time_msec,
+static void handle_pointer_motion(struct sway_seat *seat, uint32_t time_msec,
 		double dx, double dy) {
 	struct seatop_down_event *e = seat->seatop_data;
 	struct sway_container *con = e->con;
@@ -49,6 +50,28 @@ static void handle_motion(struct sway_seat *seat, uint32_t time_msec,
 	}
 }
 
+static void handle_tablet_tool_tip(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec,
+		enum wlr_tablet_tool_tip_state state) {
+	if (state == WLR_TABLET_TOOL_TIP_UP) {
+		wlr_tablet_v2_tablet_tool_notify_up(tool->tablet_v2_tool);
+		seatop_begin_default(seat);
+	}
+}
+
+static void handle_tablet_tool_motion(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec, double dx, double dy) {
+	struct seatop_down_event *e = seat->seatop_data;
+	struct sway_container *con = e->con;
+	if (seat_is_input_allowed(seat, con->view->surface)) {
+		double moved_x = seat->cursor->cursor->x - e->ref_lx;
+		double moved_y = seat->cursor->cursor->y - e->ref_ly;
+		double sx = e->ref_con_lx + moved_x;
+		double sy = e->ref_con_ly + moved_y;
+		wlr_tablet_v2_tablet_tool_notify_motion(tool->tablet_v2_tool, sx, sy);
+	}
+}
+
 static void handle_unref(struct sway_seat *seat, struct sway_container *con) {
 	struct seatop_down_event *e = seat->seatop_data;
 	if (e->con == con) {
@@ -57,9 +80,11 @@ static void handle_unref(struct sway_seat *seat, struct sway_container *con) {
 }
 
 static const struct sway_seatop_impl seatop_impl = {
-	.axis = handle_axis,
 	.button = handle_button,
-	.motion = handle_motion,
+	.pointer_motion = handle_pointer_motion,
+	.pointer_axis = handle_pointer_axis,
+	.tablet_tool_tip = handle_tablet_tool_tip,
+	.tablet_tool_motion = handle_tablet_tool_motion,
 	.unref = handle_unref,
 	.allow_set_cursor = true,
 };
