@@ -206,7 +206,7 @@ static void handle_motion_postthreshold(struct sway_seat *seat) {
 	desktop_damage_box(&e->drop_box);
 }
 
-static void handle_motion(struct sway_seat *seat, uint32_t time_msec,
+static void handle_pointer_motion(struct sway_seat *seat, uint32_t time_msec,
 		double dx, double dy) {
 	struct seatop_move_tiling_event *e = seat->seatop_data;
 	if (e->threshold_reached) {
@@ -223,13 +223,7 @@ static bool is_parallel(enum sway_container_layout layout,
 	return layout_is_horiz == edge_is_horiz;
 }
 
-static void handle_button(struct sway_seat *seat, uint32_t time_msec,
-		struct wlr_input_device *device, uint32_t button,
-		enum wlr_button_state state) {
-	if (seat->cursor->pressed_button_count != 0) {
-		return;
-	}
-
+static void finalize_move(struct sway_seat *seat) {
 	struct seatop_move_tiling_event *e = seat->seatop_data;
 
 	if (!e->target_node) {
@@ -251,7 +245,6 @@ static void handle_button(struct sway_seat *seat, uint32_t time_msec,
 		container_detach(con);
 	}
 
-
 	// Moving container into empty workspace
 	if (target_node->type == N_WORKSPACE && edge == WLR_EDGE_NONE) {
 		workspace_add_tiling(new_ws, con);
@@ -303,6 +296,22 @@ static void handle_button(struct sway_seat *seat, uint32_t time_msec,
 	seatop_begin_default(seat);
 }
 
+static void handle_button(struct sway_seat *seat, uint32_t time_msec,
+		struct wlr_input_device *device, uint32_t button,
+		enum wlr_button_state state) {
+	if (seat->cursor->pressed_button_count == 0) {
+		finalize_move(seat);
+	}
+}
+
+static void handle_tablet_tool_tip(struct sway_seat *seat,
+		struct sway_tablet_tool *tool, uint32_t time_msec,
+		enum wlr_tablet_tool_tip_state state) {
+	if (state == WLR_TABLET_TOOL_TIP_UP) {
+		finalize_move(seat);
+	}
+}
+
 static void handle_unref(struct sway_seat *seat, struct sway_container *con) {
 	struct seatop_move_tiling_event *e = seat->seatop_data;
 	if (e->target_node == &con->node) { // Drop target
@@ -315,7 +324,8 @@ static void handle_unref(struct sway_seat *seat, struct sway_container *con) {
 
 static const struct sway_seatop_impl seatop_impl = {
 	.button = handle_button,
-	.motion = handle_motion,
+	.pointer_motion = handle_pointer_motion,
+	.tablet_tool_tip = handle_tablet_tool_tip,
 	.unref = handle_unref,
 	.render = handle_render,
 };
@@ -337,7 +347,7 @@ void seatop_begin_move_tiling_threshold(struct sway_seat *seat,
 	seat->seatop_data = e;
 
 	container_raise_floating(con);
-	wlr_seat_pointer_clear_focus(seat->wlr_seat);
+	wlr_seat_pointer_notify_clear_focus(seat->wlr_seat);
 }
 
 void seatop_begin_move_tiling(struct sway_seat *seat,
