@@ -7,6 +7,7 @@
 #include "sway/tree/arrange.h"
 #include "sway/tree/view.h"
 #include "sway/tree/workspace.h"
+#include "sway/tree/container.h"
 
 struct seatop_resize_floating_event {
 	struct sway_container *con;
@@ -25,7 +26,7 @@ static void handle_button(struct sway_seat *seat, uint32_t time_msec,
 	}
 }
 
-static void handle_motion(struct sway_seat *seat, uint32_t time_msec,
+static void handle_pointer_motion(struct sway_seat *seat, uint32_t time_msec,
 		double dx, double dy) {
 	struct seatop_resize_floating_event *e = seat->seatop_data;
 	struct sway_container *con = e->con;
@@ -55,22 +56,36 @@ static void handle_motion(struct sway_seat *seat, uint32_t time_msec,
 		grow_height = e->ref_height * max_multiplier;
 	}
 
+	struct sway_container_state *state = &con->current;
+	double border_width = 0.0;
+	if (con->current.border == B_NORMAL || con->current.border == B_PIXEL) {
+		border_width = state->border_thickness * 2;
+	}
+	double border_height = 0.0;
+	if (con->current.border == B_NORMAL) {
+		border_height += container_titlebar_height();
+		border_height += state->border_thickness;
+	} else if (con->current.border == B_PIXEL) {
+		border_height += state->border_thickness * 2;
+	}
+
 	// Determine new width/height, and accommodate for floating min/max values
 	double width = e->ref_width + grow_width;
 	double height = e->ref_height + grow_height;
 	int min_width, max_width, min_height, max_height;
 	floating_calculate_constraints(&min_width, &max_width,
 			&min_height, &max_height);
-	width = fmax(min_width, fmin(width, max_width));
-	height = fmax(min_height, fmin(height, max_height));
+	width = fmax(min_width + border_width, fmin(width, max_width));
+	height = fmax(min_height + border_height, fmin(height, max_height));
 
 	// Apply the view's min/max size
 	if (con->view) {
 		double view_min_width, view_max_width, view_min_height, view_max_height;
 		view_get_constraints(con->view, &view_min_width, &view_max_width,
 				&view_min_height, &view_max_height);
-		width = fmax(view_min_width, fmin(width, view_max_width));
-		height = fmax(view_min_height, fmin(height, view_max_height));
+		width = fmax(view_min_width + border_width, fmin(width, view_max_width));
+		height = fmax(view_min_height + border_height, fmin(height, view_max_height));
+
 	}
 
 	// Recalculate these, in case we hit a min/max limit
@@ -125,7 +140,7 @@ static void handle_unref(struct sway_seat *seat, struct sway_container *con) {
 
 static const struct sway_seatop_impl seatop_impl = {
 	.button = handle_button,
-	.motion = handle_motion,
+	.pointer_motion = handle_pointer_motion,
 	.unref = handle_unref,
 };
 
@@ -160,5 +175,5 @@ void seatop_begin_resize_floating(struct sway_seat *seat,
 	const char *image = edge == WLR_EDGE_NONE ?
 		"se-resize" : wlr_xcursor_get_resize_name(edge);
 	cursor_set_image(seat->cursor, image, NULL);
-	wlr_seat_pointer_clear_focus(seat->wlr_seat);
+	wlr_seat_pointer_notify_clear_focus(seat->wlr_seat);
 }
