@@ -18,6 +18,7 @@
 #include <wlr/types/wlr_output.h>
 #include <xkbcommon/xkbcommon.h>
 #include "wlr-layer-shell-unstable-v1-protocol.h"
+#include "sway/desktop/idle_inhibit_v1.h"
 
 static const int i3_output_id = INT32_MAX;
 static const int i3_scratch_id = INT32_MAX - 1;
@@ -68,19 +69,89 @@ static const char *ipc_json_output_transform_description(enum wl_output_transfor
 	case WL_OUTPUT_TRANSFORM_NORMAL:
 		return "normal";
 	case WL_OUTPUT_TRANSFORM_90:
-		return "90";
+		// Sway uses clockwise transforms, while WL_OUTPUT_TRANSFORM_* describes
+		// anti-clockwise transforms.
+		return "270";
 	case WL_OUTPUT_TRANSFORM_180:
 		return "180";
 	case WL_OUTPUT_TRANSFORM_270:
-		return "270";
+		// Transform also inverted here.
+		return "90";
 	case WL_OUTPUT_TRANSFORM_FLIPPED:
 		return "flipped";
 	case WL_OUTPUT_TRANSFORM_FLIPPED_90:
-		return "flipped-90";
+		// Inverted.
+		return "flipped-270";
 	case WL_OUTPUT_TRANSFORM_FLIPPED_180:
 		return "flipped-180";
 	case WL_OUTPUT_TRANSFORM_FLIPPED_270:
-		return "flipped-270";
+		// Inverted.
+		return "flipped-90";
+	}
+	return NULL;
+}
+
+static const char *ipc_json_output_adaptive_sync_status_description(
+		enum wlr_output_adaptive_sync_status status) {
+	switch (status) {
+	case WLR_OUTPUT_ADAPTIVE_SYNC_DISABLED:
+		return "disabled";
+	case WLR_OUTPUT_ADAPTIVE_SYNC_ENABLED:
+		return "enabled";
+	case WLR_OUTPUT_ADAPTIVE_SYNC_UNKNOWN:
+		return "unknown";
+	}
+	return NULL;
+}
+
+#if HAVE_XWAYLAND
+static const char *ipc_json_xwindow_type_description(struct sway_view *view) {
+	struct wlr_xwayland_surface *surface = view->wlr_xwayland_surface;
+	struct sway_xwayland *xwayland = &server.xwayland;
+
+	for (size_t i = 0; i < surface->window_type_len; ++i) {
+		xcb_atom_t type = surface->window_type[i];
+		if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_NORMAL]) {
+			return "normal";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_DIALOG]) {
+			return "dialog";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_UTILITY]) {
+			return "utility";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_TOOLBAR]) {
+			return "toolbar";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_SPLASH]) {
+			return "splash";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_MENU]) {
+			return "menu";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_DROPDOWN_MENU]) {
+			return "dropdown_menu";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_POPUP_MENU]) {
+			return "popup_menu";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_TOOLTIP]) {
+			return "tooltip";
+		} else if (type == xwayland->atoms[NET_WM_WINDOW_TYPE_NOTIFICATION]) {
+			return "notification";
+		} else {
+			return "unknown";
+		}
+	}
+
+	return "unknown";
+}
+#endif
+
+static const char *ipc_json_user_idle_inhibitor_description(enum sway_idle_inhibit_mode mode) {
+	switch (mode) {
+	case INHIBIT_IDLE_FOCUS:
+		return "focus";
+	case INHIBIT_IDLE_FULLSCREEN:
+		return "fullscreen";
+	case INHIBIT_IDLE_OPEN:
+		return "open";
+	case INHIBIT_IDLE_VISIBLE:
+		return "visible";
+	case INHIBIT_IDLE_APPLICATION:
+		return NULL;
 	}
 	return NULL;
 }
@@ -147,6 +218,9 @@ static json_object *ipc_json_create_node(int id, char *name,
 	json_object_object_add(object, "geometry", ipc_json_create_empty_rect());
 	json_object_object_add(object, "window", NULL);
 	json_object_object_add(object, "urgent", json_object_new_boolean(false));
+	json_object_object_add(object, "marks", json_object_new_array());
+	json_object_object_add(object, "fullscreen_mode", json_object_new_int(0));
+	json_object_object_add(object, "nodes", json_object_new_array());
 	json_object_object_add(object, "floating_nodes", json_object_new_array());
 	json_object_object_add(object, "sticky", json_object_new_boolean(false));
 
@@ -183,6 +257,11 @@ static void ipc_json_describe_output(struct sway_output *output,
 	json_object_object_add(object, "transform",
 		json_object_new_string(
 			ipc_json_output_transform_description(wlr_output->transform)));
+	const char *adaptive_sync_status =
+		ipc_json_output_adaptive_sync_status_description(
+			wlr_output->adaptive_sync_status);
+	json_object_object_add(object, "adaptive_sync_status",
+		json_object_new_string(adaptive_sync_status));
 
 	struct sway_workspace *ws = output_get_active_workspace(output);
 	if (!sway_assert(ws, "Expected output to have a workspace")) {
@@ -292,6 +371,7 @@ static json_object *ipc_json_describe_scratchpad_output(void) {
 
 	json_object *workspace = ipc_json_create_node(i3_scratch_id,
 				"__i3_scratch", false, workspace_focus, &box);
+	json_object_object_add(workspace, "fullscreen_mode", json_object_new_int(1));
 	json_object_object_add(workspace, "type",
 			json_object_new_string("workspace"));
 
@@ -326,9 +406,21 @@ static json_object *ipc_json_describe_scratchpad_output(void) {
 
 static void ipc_json_describe_workspace(struct sway_workspace *workspace,
 		json_object *object) {
-	int num = isdigit(workspace->name[0]) ? atoi(workspace->name) : -1;
-
+	int num;
+	if (isdigit(workspace->name[0])) {
+		errno = 0;
+		char *endptr = NULL;
+		long long parsed_num = strtoll(workspace->name, &endptr, 10);
+		if (errno != 0 || parsed_num > INT32_MAX || parsed_num < 0 || endptr == workspace->name) {
+			num = -1;
+		} else {
+			num = (int) parsed_num;
+		}
+	} else {
+		num = -1;
+	}
 	json_object_object_add(object, "num", json_object_new_int(num));
+	json_object_object_add(object, "fullscreen_mode", json_object_new_int(1));
 	json_object_object_add(object, "output", workspace->output ?
 			json_object_new_string(workspace->output->wlr_output->name) : NULL);
 	json_object_object_add(object, "type", json_object_new_string("workspace"));
@@ -401,14 +493,6 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 	bool visible = view_is_visible(c->view);
 	json_object_object_add(object, "visible", json_object_new_boolean(visible));
 
-	json_object *marks = json_object_new_array();
-	list_t *con_marks = c->marks;
-	for (int i = 0; i < con_marks->length; ++i) {
-		json_object_array_add(marks, json_object_new_string(con_marks->items[i]));
-	}
-
-	json_object_object_add(object, "marks", marks);
-
 	struct wlr_box window_box = {
 		c->content_x - c->x,
 		(c->current.border == B_PIXEL) ? c->current.border_thickness : 0,
@@ -423,6 +507,38 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 
 	json_object_object_add(object, "max_render_time", json_object_new_int(c->view->max_render_time));
 
+	json_object_object_add(object, "shell", json_object_new_string(view_get_shell(c->view)));
+
+	json_object_object_add(object, "inhibit_idle",
+		json_object_new_boolean(view_inhibit_idle(c->view)));
+
+	json_object *idle_inhibitors = json_object_new_object();
+
+	struct sway_idle_inhibitor_v1 *user_inhibitor =
+		sway_idle_inhibit_v1_user_inhibitor_for_view(c->view);
+
+	if (user_inhibitor) {
+		json_object_object_add(idle_inhibitors, "user",
+			json_object_new_string(
+				ipc_json_user_idle_inhibitor_description(user_inhibitor->mode)));
+	} else {
+		json_object_object_add(idle_inhibitors, "user",
+			json_object_new_string("none"));
+	}
+
+	struct sway_idle_inhibitor_v1 *application_inhibitor =
+		sway_idle_inhibit_v1_application_inhibitor_for_view(c->view);
+
+	if (application_inhibitor) {
+		json_object_object_add(idle_inhibitors, "application",
+			json_object_new_string("enabled"));
+	} else {
+		json_object_object_add(idle_inhibitors, "application",
+			json_object_new_string("none"));
+	}
+
+	json_object_object_add(object, "idle_inhibitors", idle_inhibitors);
+
 #if HAVE_XWAYLAND
 	if (c->view->type == SWAY_VIEW_XWAYLAND) {
 		json_object_object_add(object, "window",
@@ -452,6 +568,13 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 			json_object_object_add(window_props, "window_role", json_object_new_string(role));
 		}
 
+		uint32_t window_type = view_get_window_type(c->view);
+		if (window_type) {
+			json_object_object_add(window_props, "window_type",
+				json_object_new_string(
+					ipc_json_xwindow_type_description(c->view)));
+		}
+
 		json_object_object_add(object, "window_properties", window_props);
 	}
 #endif
@@ -503,6 +626,14 @@ static void ipc_json_describe_container(struct sway_container *c, json_object *o
 	get_deco_rect(c, &deco_box);
 	json_object_object_add(object, "deco_rect", ipc_json_create_rect(&deco_box));
 
+	json_object *marks = json_object_new_array();
+	list_t *con_marks = c->marks;
+	for (int i = 0; i < con_marks->length; ++i) {
+		json_object_array_add(marks, json_object_new_string(con_marks->items[i]));
+	}
+
+	json_object_object_add(object, "marks", marks);
+
 	if (c->view) {
 		ipc_json_describe_view(c, object);
 	}
@@ -1130,3 +1261,10 @@ json_object *ipc_json_describe_bar_config(struct bar_config *bar) {
 #endif
 	return json;
 }
+
+json_object *ipc_json_get_binding_mode(void) {
+	json_object *current_mode = json_object_new_object();
+	json_object_object_add(current_mode, "name",
+			json_object_new_string(config->current_mode->name));
+	return current_mode;
+}
