@@ -47,7 +47,6 @@ struct ipc_client {
 	struct wl_event_source *writable_event_source;
 	struct sway_server *server;
 	int fd;
-	uint32_t security_policy;
 	enum ipc_command_type subscribed_events;
 	size_t write_buffer_len;
 	size_t write_buffer_size;
@@ -421,7 +420,8 @@ void ipc_event_binding(struct sway_binding *binding) {
 	json_object *symbols = json_object_new_array();
 	json_object *symbol = NULL;
 
-	if (binding->type == BINDING_KEYCODE) { // bindcode: populate input_codes
+	switch (binding->type) {
+	case BINDING_KEYCODE:; // bindcode: populate input_codes
 		uint32_t keycode;
 		for (int i = 0; i < binding->keys->length; ++i) {
 			keycode = *(uint32_t *)binding->keys->items[i];
@@ -430,7 +430,11 @@ void ipc_event_binding(struct sway_binding *binding) {
 				input_code = keycode;
 			}
 		}
-	} else { // bindsym/mouse: populate symbols
+		break;
+
+	case BINDING_KEYSYM:
+	case BINDING_MOUSESYM:
+	case BINDING_MOUSECODE:; // bindsym/mouse: populate symbols
 		uint32_t keysym;
 		char buffer[64];
 		for (int i = 0; i < binding->keys->length; ++i) {
@@ -451,6 +455,14 @@ void ipc_event_binding(struct sway_binding *binding) {
 				json_object_array_add(symbols, str);
 			}
 		}
+		break;
+
+	default:
+		sway_log(SWAY_DEBUG, "Unsupported ipc binding event");
+		json_object_put(input_codes);
+		json_object_put(symbols);
+		json_object_put(json_binding);
+		return; // do not send any event
 	}
 
 	json_object_object_add(json_binding, "input_codes", input_codes);
@@ -864,6 +876,16 @@ void ipc_client_handle_command(struct ipc_client *client, uint32_t payload_lengt
 		goto exit_cleanup;
 	}
 
+	case IPC_GET_BINDING_STATE:
+	{
+		json_object *current_mode = ipc_json_get_binding_mode();
+		const char *json_string = json_object_to_json_string(current_mode);
+		ipc_send_reply(client, payload_type, json_string,
+			(uint32_t)strlen(json_string));
+		json_object_put(current_mode); // free
+		goto exit_cleanup;
+	}
+
 	case IPC_GET_CONFIG:
 	{
 		json_object *json = json_object_new_object();
@@ -910,7 +932,8 @@ bool ipc_send_reply(struct ipc_client *client, enum ipc_command_type payload_typ
 	}
 
 	if (client->write_buffer_size > 4e6) { // 4 MB
-		sway_log(SWAY_ERROR, "Client write buffer too big, disconnecting client");
+		sway_log(SWAY_ERROR, "Client write buffer too big (%zu), disconnecting client",
+				client->write_buffer_size);
 		ipc_client_disconnect(client);
 		return false;
 	}
