@@ -27,6 +27,7 @@
 static bool terminate_request = false;
 static int exit_value = 0;
 struct sway_server server = {0};
+struct sway_debug debug = {0};
 
 void sway_terminate(int exit_code) {
 	if (!server.wl_display) {
@@ -185,12 +186,17 @@ static void log_kernel(void) {
 
 static bool drop_permissions(void) {
 	if (getuid() != geteuid() || getgid() != getegid()) {
-		if (setuid(getuid()) != 0 || setgid(getgid()) != 0) {
-			sway_log(SWAY_ERROR, "Unable to drop root, refusing to start");
+		// Set the gid and uid in the correct order.
+		if (setgid(getgid()) != 0) {
+			sway_log(SWAY_ERROR, "Unable to drop root group, refusing to start");
+			return false;
+		}
+		if (setuid(getuid()) != 0) {
+			sway_log(SWAY_ERROR, "Unable to drop root user, refusing to start");
 			return false;
 		}
 	}
-	if (setuid(0) != -1) {
+	if (setgid(0) != -1 || setuid(0) != -1) {
 		sway_log(SWAY_ERROR, "Unable to drop root (we shouldn't be able to "
 			"restore it after setuid), refusing to start");
 		return false;
@@ -309,7 +315,7 @@ int main(int argc, char **argv) {
 	if (debug) {
 		sway_log_init(SWAY_DEBUG, sway_terminate);
 		wlr_log_init(WLR_DEBUG, NULL);
-	} else if (verbose || validate) {
+	} else if (verbose) {
 		sway_log_init(SWAY_INFO, sway_terminate);
 		wlr_log_init(WLR_INFO, NULL);
 	} else {
@@ -317,6 +323,7 @@ int main(int argc, char **argv) {
 		wlr_log_init(WLR_ERROR, NULL);
 	}
 
+	sway_log(SWAY_INFO, "Sway version " SWAY_VERSION);
 	log_kernel();
 	log_distro();
 	log_env();
