@@ -4,6 +4,8 @@
 #include <stdlib.h>
 #include <wayland-server-core.h>
 #include <wlr/backend.h>
+#include <wlr/backend/headless.h>
+#include <wlr/backend/multi.h>
 #include <wlr/backend/noop.h>
 #include <wlr/backend/session.h>
 #include <wlr/render/wlr_renderer.h>
@@ -20,6 +22,7 @@
 #include <wlr/types/wlr_screencopy_v1.h>
 #include <wlr/types/wlr_server_decoration.h>
 #include <wlr/types/wlr_tablet_v2.h>
+#include <wlr/types/wlr_viewporter.h>
 #include <wlr/types/wlr_xcursor_manager.h>
 #include <wlr/types/wlr_xdg_decoration_v1.h>
 #include <wlr/types/wlr_xdg_output_v1.h>
@@ -41,7 +44,6 @@ bool server_privileged_prepare(struct sway_server *server) {
 	server->wl_display = wl_display_create();
 	server->wl_event_loop = wl_display_get_event_loop(server->wl_display);
 	server->backend = wlr_backend_autocreate(server->wl_display, NULL);
-	server->noop_backend = wlr_noop_backend_create(server->wl_display);
 
 	if (!server->backend) {
 		sway_log(SWAY_ERROR, "Unable to create backend");
@@ -132,10 +134,22 @@ bool server_init(struct sway_server *server) {
 	wl_signal_add(&server->output_manager_v1->events.test,
 		&server->output_manager_test);
 
+	server->output_power_manager_v1 =
+		wlr_output_power_manager_v1_create(server->wl_display);
+	server->output_power_manager_set_mode.notify =
+		handle_output_power_manager_set_mode;
+	wl_signal_add(&server->output_power_manager_v1->events.set_mode,
+		&server->output_power_manager_set_mode);
+	server->input_method = wlr_input_method_manager_v2_create(server->wl_display);
+	server->text_input = wlr_text_input_manager_v3_create(server->wl_display);
+	server->foreign_toplevel_manager =
+		wlr_foreign_toplevel_manager_v1_create(server->wl_display);
+
 	wlr_export_dmabuf_manager_v1_create(server->wl_display);
 	wlr_screencopy_manager_v1_create(server->wl_display);
 	wlr_data_control_manager_v1_create(server->wl_display);
 	wlr_primary_selection_v1_device_manager_create(server->wl_display);
+	wlr_viewporter_create(server->wl_display);
 
 	server->socket = wl_display_add_socket_auto(server->wl_display);
 	if (!server->socket) {
@@ -144,9 +158,15 @@ bool server_init(struct sway_server *server) {
 		return false;
 	}
 
+	server->noop_backend = wlr_noop_backend_create(server->wl_display);
+
 	struct wlr_output *wlr_output = wlr_noop_add_output(server->noop_backend);
 	root->noop_output = output_create(wlr_output);
 
+	server->headless_backend =
+		wlr_headless_backend_create_with_renderer(server->wl_display, renderer);
+	wlr_multi_backend_add(server->backend, server->headless_backend);
+
 	// This may have been set already via -Dtxn-timeout
 	if (!server->txn_timeout_ms) {
 		server->txn_timeout_ms = 200;
@@ -180,16 +200,21 @@ bool server_start(struct sway_server *server) {
 		server->xwayland.wlr_xwayland =
 			wlr_xwayland_create(server->wl_display, server->compositor,
 					config->xwayland == XWAYLAND_MODE_LAZY);
-		wl_signal_add(&server->xwayland.wlr_xwayland->events.new_surface,
-			&server->xwayland_surface);
-		server->xwayland_surface.notify = handle_xwayland_surface;
-		wl_signal_add(&server->xwayland.wlr_xwayland->events.ready,
-			&server->xwayland_ready);
-		server->xwayland_ready.notify = handle_xwayland_ready;
+		if (!server->xwayland.wlr_xwayland) {
+			sway_log(SWAY_ERROR, "Failed to start Xwayland");
+			unsetenv("DISPLAY");
+		} else {
+			wl_signal_add(&server->xwayland.wlr_xwayland->events.new_surface,
+				&server->xwayland_surface);
+			server->xwayland_surface.notify = handle_xwayland_surface;
+			wl_signal_add(&server->xwayland.wlr_xwayland->events.ready,
+				&server->xwayland_ready);
+			server->xwayland_ready.notify = handle_xwayland_ready;
 
-		setenv("DISPLAY", server->xwayland.wlr_xwayland->display_name, true);
+			setenv("DISPLAY", server->xwayland.wlr_xwayland->display_name, true);
 
-		/* xcursor configured by the default seat */
+			/* xcursor configured by the default seat */
+		}
 	}
 #endif
 
