@@ -208,7 +208,8 @@ static struct sway_container *container_at_tabbed(struct sway_node *parent,
 		struct wlr_surface **surface, double *sx, double *sy) {
 	struct wlr_box box;
 	node_get_box(parent, &box);
-	if (ly < box.y || ly > box.y + box.height) {
+	if (lx < box.x || lx > box.x + box.width ||
+			ly < box.y || ly > box.y + box.height) {
 		return NULL;
 	}
 	struct sway_seat *seat = input_manager_current_seat();
@@ -242,7 +243,8 @@ static struct sway_container *container_at_stacked(struct sway_node *parent,
 		struct wlr_surface **surface, double *sx, double *sy) {
 	struct wlr_box box;
 	node_get_box(parent, &box);
-	if (ly < box.y || ly > box.y + box.height) {
+	if (lx < box.x || lx > box.x + box.width ||
+			ly < box.y || ly > box.y + box.height) {
 		return NULL;
 	}
 	struct sway_seat *seat = input_manager_current_seat();
@@ -272,14 +274,10 @@ static struct sway_container *container_at_linear(struct sway_node *parent,
 	list_t *children = node_get_children(parent);
 	for (int i = 0; i < children->length; ++i) {
 		struct sway_container *child = children->items[i];
-		struct wlr_box box = {
-			.x = child->x,
-			.y = child->y,
-			.width = child->width,
-			.height = child->height,
-		};
-		if (wlr_box_contains_point(&box, lx, ly)) {
-			return tiling_container_at(&child->node, lx, ly, surface, sx, sy);
+		struct sway_container *container =
+			tiling_container_at(&child->node, lx, ly, surface, sx, sy);
+		if (container) {
+			return container;
 		}
 	}
 	return NULL;
@@ -287,7 +285,10 @@ static struct sway_container *container_at_linear(struct sway_node *parent,
 
 static struct sway_container *floating_container_at(double lx, double ly,
 		struct wlr_surface **surface, double *sx, double *sy) {
-	for (int i = 0; i < root->outputs->length; ++i) {
+	// For outputs with floating containers that overhang the output bounds,
+	// those at the end of the output list appear on top of floating
+	// containers from other outputs, so iterate the list in reverse.
+	for (int i = root->outputs->length - 1; i >= 0; --i) {
 		struct sway_output *output = root->outputs->items[i];
 		for (int j = 0; j < output->workspaces->length; ++j) {
 			struct sway_workspace *ws = output->workspaces->items[j];
@@ -298,15 +299,10 @@ static struct sway_container *floating_container_at(double lx, double ly,
 			// reverse.
 			for (int k = ws->floating->length - 1; k >= 0; --k) {
 				struct sway_container *floater = ws->floating->items[k];
-				struct wlr_box box = {
-					.x = floater->x,
-					.y = floater->y,
-					.width = floater->width,
-					.height = floater->height,
-				};
-				if (wlr_box_contains_point(&box, lx, ly)) {
-					return tiling_container_at(&floater->node, lx, ly,
-							surface, sx, sy);
+				struct sway_container *container =
+					tiling_container_at(&floater->node, lx, ly, surface, sx, sy);
+				if (container) {
+					return container;
 				}
 			}
 		}
@@ -314,12 +310,34 @@ static struct sway_container *floating_container_at(double lx, double ly,
 	return NULL;
 }
 
+struct sway_container *view_container_at(struct sway_node *parent,
+		double lx, double ly,
+		struct wlr_surface **surface, double *sx, double *sy) {
+	if (!sway_assert(node_is_view(parent), "Expected a view")) {
+		return NULL;
+	}
+
+	struct sway_container *container = parent->sway_container;
+	struct wlr_box box = {
+			.x = container->x,
+			.y = container->y,
+			.width = container->width,
+			.height = container->height,
+	};
+
+	if (wlr_box_contains_point(&box, lx, ly)) {
+		surface_at_view(parent->sway_container, lx, ly, surface, sx, sy);
+		return container;
+	}
+
+	return NULL;
+}
+
 struct sway_container *tiling_container_at(struct sway_node *parent,
 		double lx, double ly,
 		struct wlr_surface **surface, double *sx, double *sy) {
 	if (node_is_view(parent)) {
-		surface_at_view(parent->sway_container, lx, ly, surface, sx, sy);
-		return parent->sway_container;
+		return view_container_at(parent, lx, ly, surface, sx, sy);
 	}
 	if (!node_get_children(parent)) {
 		return NULL;
@@ -454,8 +472,13 @@ static void update_title_texture(struct sway_container *con,
 	cairo_set_antialias(c, CAIRO_ANTIALIAS_BEST);
 	cairo_font_options_t *fo = cairo_font_options_create();
 	cairo_font_options_set_hint_style(fo, CAIRO_HINT_STYLE_FULL);
-	cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_SUBPIXEL);
-	cairo_font_options_set_subpixel_order(fo, to_cairo_subpixel_order(output->wlr_output->subpixel));
+	if (output->wlr_output->subpixel == WL_OUTPUT_SUBPIXEL_NONE) {
+		cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_GRAY);
+	} else {
+		cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_SUBPIXEL);
+		cairo_font_options_set_subpixel_order(fo,
+			to_cairo_subpixel_order(output->wlr_output->subpixel));
+	}
 	cairo_set_font_options(c, fo);
 	get_text_size(c, config->font, &width, NULL, NULL, scale,
 			config->pango_markup, "%s", con->formatted_title);
@@ -1085,11 +1108,17 @@ void container_set_fullscreen(struct sway_container *con,
 	}
 }
 
-bool container_is_floating_or_child(struct sway_container *container) {
+struct sway_container *container_toplevel_ancestor(
+		struct sway_container *container) {
 	while (container->parent) {
 		container = container->parent;
 	}
-	return container_is_floating(container);
+
+	return container;
+}
+
+bool container_is_floating_or_child(struct sway_container *container) {
+	return container_is_floating(container_toplevel_ancestor(container));
 }
 
 bool container_is_fullscreen_or_child(struct sway_container *container) {
@@ -1139,6 +1168,10 @@ void container_discover_outputs(struct sway_container *con) {
 			if (con->view) {
 				view_for_each_surface(con->view,
 						surface_send_enter_iterator, output->wlr_output);
+				if (con->view->foreign_toplevel) {
+					wlr_foreign_toplevel_handle_v1_output_enter(
+							con->view->foreign_toplevel, output->wlr_output);
+				}
 			}
 			list_add(con->outputs, output);
 		} else if (!intersects && index != -1) {
@@ -1147,6 +1180,10 @@ void container_discover_outputs(struct sway_container *con) {
 			if (con->view) {
 				view_for_each_surface(con->view,
 					surface_send_leave_iterator, output->wlr_output);
+				if (con->view->foreign_toplevel) {
+					wlr_foreign_toplevel_handle_v1_output_leave(
+							con->view->foreign_toplevel, output->wlr_output);
+				}
 			}
 			list_del(con->outputs, index);
 		}
@@ -1292,6 +1329,14 @@ void container_detach(struct sway_container *child) {
 		container_update_representation(old_parent);
 		node_set_dirty(&old_parent->node);
 	} else if (old_workspace) {
+		// We may have removed the last tiling child from the workspace. If the
+		// workspace layout was e.g. tabbed, then at this point it may be just
+		// H[]. So, reset it to the default (e.g. T[]) for next time.
+		if (!old_workspace->tiling->length) {
+			old_workspace->layout =
+				output_get_default_layout(old_workspace->output);
+		}
+
 		workspace_update_representation(old_workspace);
 		node_set_dirty(&old_workspace->node);
 	}
@@ -1514,10 +1559,7 @@ void container_update_marks_textures(struct sway_container *con) {
 
 void container_raise_floating(struct sway_container *con) {
 	// Bring container to front by putting it at the end of the floating list.
-	struct sway_container *floater = con;
-	while (floater->parent) {
-		floater = floater->parent;
-	}
+	struct sway_container *floater = container_toplevel_ancestor(con);
 	if (container_is_floating(floater) && floater->workspace) {
 		list_move_to_end(floater->workspace->floating, floater);
 		node_set_dirty(&floater->workspace->node);
@@ -1529,8 +1571,6 @@ bool container_is_scratchpad_hidden(struct sway_container *con) {
 }
 
 bool container_is_scratchpad_hidden_or_child(struct sway_container *con) {
-	while (con->parent) {
-		con = con->parent;
-	}
+	con = container_toplevel_ancestor(con);
 	return con->scratchpad && !con->workspace;
 }
