@@ -102,26 +102,20 @@ struct sway_output *output_create(struct wlr_output *wlr_output) {
 	output->workspaces = create_list();
 	output->current.workspaces = create_list();
 
-	return output;
-}
-
-bool output_enable(struct sway_output *output, struct output_config *oc) {
-	if (!sway_assert(!output->enabled, "output is already enabled")) {
-		return false;
-	}
-	struct wlr_output *wlr_output = output->wlr_output;
 	size_t len = sizeof(output->layers) / sizeof(output->layers[0]);
 	for (size_t i = 0; i < len; ++i) {
 		wl_list_init(&output->layers[i]);
 	}
 
-	output->enabled = true;
-	if (!apply_output_config(oc, output)) {
-		output->enabled = false;
-		return false;
-	}
+	return output;
+}
 
-	output->configured = true;
+void output_enable(struct sway_output *output) {
+	if (!sway_assert(!output->enabled, "output is already enabled")) {
+		return;
+	}
+	struct wlr_output *wlr_output = output->wlr_output;
+	output->enabled = true;
 	list_add(root->outputs, output);
 
 	restore_workspaces(output);
@@ -156,8 +150,6 @@ bool output_enable(struct sway_output *output, struct output_config *oc) {
 
 	arrange_layers(output);
 	arrange_root();
-
-	return true;
 }
 
 static void evacuate_sticky(struct sway_workspace *old_ws,
@@ -259,6 +251,11 @@ void output_disable(struct sway_output *output) {
 	if (!sway_assert(output->enabled, "Expected an enabled output")) {
 		return;
 	}
+	int index = list_find(root->outputs, output);
+	if (!sway_assert(index >= 0, "Output not found in root node")) {
+		return;
+	}
+
 	sway_log(SWAY_DEBUG, "Disabling output '%s'", output->wlr_output->name);
 	wl_signal_emit(&output->events.destroy, output);
 
@@ -266,14 +263,17 @@ void output_disable(struct sway_output *output) {
 
 	root_for_each_container(untrack_output, output);
 
-	int index = list_find(root->outputs, output);
 	list_del(root->outputs, index);
 
 	output->enabled = false;
-	output->configured = false;
 	output->current_mode = NULL;
 
 	arrange_root();
+
+	// Reconfigure all devices, since devices with map_to_output directives for
+	// an output that goes offline should stop sending events as long as the
+	// output remains offline.
+	input_manager_configure_all_inputs();
 }
 
 void output_begin_destroy(struct sway_output *output) {
