@@ -40,7 +40,7 @@ void ipc_send_workspace_command(struct swaybar *bar, const char *ws) {
 		command[d++] = ws[i];
 	}
 
-	ipc_single_command(bar->ipc_socketfd, IPC_COMMAND, command, &size);
+	free(ipc_single_command(bar->ipc_socketfd, IPC_COMMAND, command, &size));
 	free(command);
 }
 
@@ -360,7 +360,7 @@ bool ipc_get_workspaces(struct swaybar *bar) {
 
 		wl_list_for_each(output, &bar->outputs, link) {
 			const char *ws_output = json_object_get_string(out);
-			if (strcmp(ws_output, output->name) == 0) {
+			if (ws_output != NULL && strcmp(ws_output, output->name) == 0) {
 				struct swaybar_workspace *ws =
 					calloc(1, sizeof(struct swaybar_workspace));
 				ws->num = json_object_get_int(num);
@@ -528,8 +528,10 @@ static bool handle_barconfig_update(struct swaybar *bar, const char *payload,
 		ipc_get_workspaces(bar);
 	}
 
+	bool moving_layer = strcmp(oldcfg->mode, newcfg->mode) != 0;
+
 	free_config(oldcfg);
-	determine_bar_visibility(bar, true);
+	determine_bar_visibility(bar, moving_layer);
 	return true;
 }
 
