@@ -131,11 +131,15 @@ static void render_sharp_line(cairo_t *cairo, uint32_t color,
 
 static enum hotspot_event_handling block_hotspot_callback(
 		struct swaybar_output *output, struct swaybar_hotspot *hotspot,
-		int x, int y, uint32_t button, void *data) {
+		double x, double y, uint32_t button, void *data) {
 	struct i3bar_block *block = data;
 	struct status_line *status = output->bar->status;
-	return i3bar_block_send_click(status, block, x, y, x - hotspot->x,
-			y - hotspot->y, hotspot->width, hotspot->height, button);
+	return i3bar_block_send_click(status, block, x, y,
+			x - (double)hotspot->x / output->scale,
+			y - (double)hotspot->y / output->scale,
+			(double)hotspot->width / output->scale,
+			(double)hotspot->height / output->scale,
+			output->scale, button);
 }
 
 static void i3bar_block_unref_callback(void *data) {
@@ -263,8 +267,9 @@ static uint32_t render_status_block(cairo_t *cairo,
 	} else if (strncmp(block->align, "center", 6) == 0) {
 		offset = x_pos + (width - text_width) / 2;
 	}
-	cairo_move_to(cairo, offset, height / 2.0 - text_height / 2.0);
-	uint32_t color = block->color ?  *block->color : config->colors.statusline;
+	double text_y = height / 2.0 - text_height / 2.0;
+	cairo_move_to(cairo, offset, (int)floor(text_y));
+	uint32_t color = block->color_set ? block->color : config->colors.statusline;
 	color = block->urgent ? config->colors.urgent_workspace.text : color;
 	cairo_set_source_u32(cairo, color);
 	pango_printf(cairo, config->font, output->scale,
@@ -286,7 +291,8 @@ static uint32_t render_status_block(cairo_t *cairo,
 		}
 		if (config->sep_symbol) {
 			offset = x_pos + (sep_block_width - sep_width) / 2;
-			cairo_move_to(cairo, offset, height / 2.0 - sep_height / 2.0);
+			double sep_y = height / 2.0 - sep_height / 2.0;
+			cairo_move_to(cairo, offset, (int)floor(sep_y));
 			pango_printf(cairo, config->font, output->scale, false,
 					"%s", config->sep_symbol);
 		} else {
@@ -538,7 +544,7 @@ static uint32_t render_binding_mode_indicator(cairo_t *cairo,
 
 static enum hotspot_event_handling workspace_hotspot_callback(
 		struct swaybar_output *output, struct swaybar_hotspot *hotspot,
-		int x, int y, uint32_t button, void *data) {
+		double x, double y, uint32_t button, void *data) {
 	if (button != BTN_LEFT) {
 		return HOTSPOT_PROCESS;
 	}
@@ -692,8 +698,13 @@ void render_frame(struct swaybar_output *output) {
 	cairo_set_antialias(cairo, CAIRO_ANTIALIAS_BEST);
 	cairo_font_options_t *fo = cairo_font_options_create();
 	cairo_font_options_set_hint_style(fo, CAIRO_HINT_STYLE_FULL);
-	cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_SUBPIXEL);
-	cairo_font_options_set_subpixel_order(fo, to_cairo_subpixel_order(output->subpixel));
+	if (output->subpixel == WL_OUTPUT_SUBPIXEL_NONE) {
+		cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_GRAY);
+	} else {
+		cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_SUBPIXEL);
+		cairo_font_options_set_subpixel_order(fo,
+			to_cairo_subpixel_order(output->subpixel));
+	}
 	cairo_set_font_options(cairo, fo);
 	cairo_font_options_destroy(fo);
 	cairo_save(cairo);
