@@ -190,7 +190,7 @@ static void pretty_print_output(json_object *o) {
 	json_object_object_get_ex(o, "active", &active);
 	json_object_object_get_ex(o, "current_workspace", &ws);
 	json_object *make, *model, *serial, *scale, *scale_filter, *subpixel,
-		*transform, *max_render_time;
+		*transform, *max_render_time, *adaptive_sync_status;
 	json_object_object_get_ex(o, "make", &make);
 	json_object_object_get_ex(o, "model", &model);
 	json_object_object_get_ex(o, "serial", &serial);
@@ -199,6 +199,7 @@ static void pretty_print_output(json_object *o) {
 	json_object_object_get_ex(o, "subpixel_hinting", &subpixel);
 	json_object_object_get_ex(o, "transform", &transform);
 	json_object_object_get_ex(o, "max_render_time", &max_render_time);
+	json_object_object_get_ex(o, "adaptive_sync_status", &adaptive_sync_status);
 	json_object *x, *y;
 	json_object_object_get_ex(rect, "x", &x);
 	json_object_object_get_ex(rect, "y", &y);
@@ -219,8 +220,7 @@ static void pretty_print_output(json_object *o) {
 			"  Scale filter: %s\n"
 			"  Subpixel hinting: %s\n"
 			"  Transform: %s\n"
-			"  Workspace: %s\n"
-			"  Max render time: ",
+			"  Workspace: %s\n",
 			json_object_get_string(name),
 			json_object_get_string(make),
 			json_object_get_string(model),
@@ -236,8 +236,13 @@ static void pretty_print_output(json_object *o) {
 			json_object_get_string(transform),
 			json_object_get_string(ws)
 		);
+
 		int max_render_time_int = json_object_get_int(max_render_time);
+		printf("  Max render time: ");
 		printf(max_render_time_int == 0 ? "off\n" : "%d ms\n", max_render_time_int);
+
+		printf("  Adaptive sync: %s\n",
+			json_object_get_string(adaptive_sync_status));
 	} else {
 		printf(
 			"Output %s '%s %s %s' (inactive)\n",
@@ -435,6 +440,8 @@ int main(int argc, char **argv) {
 		type = IPC_GET_VERSION;
 	} else if (strcasecmp(cmdtype, "get_binding_modes") == 0) {
 		type = IPC_GET_BINDING_MODES;
+	} else if (strcasecmp(cmdtype, "get_binding_state") == 0) {
+		type = IPC_GET_BINDING_STATE;
 	} else if (strcasecmp(cmdtype, "get_config") == 0) {
 		type = IPC_GET_CONFIG;
 	} else if (strcasecmp(cmdtype, "send_tick") == 0) {
